#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <rpc/rpc.h>
#include <unistd.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/file.h>
#include "ex1048.h"
#include "calfile.h"
#include "acqmain.h"
#include "libex1048.h"
#include "dmmlib.h"
#include "errlog.h"

/*
 * !!!! many readings are lacking their delays
 */

/*
  !!! HACK !!!
  
  why isn't the header file stdlib.h declaring this properly?
*/
extern float strtof (__const char *__restrict __nptr,
		     char **__restrict __endptr) __THROW;

#define CH_TC_MIN 0
#define CH_TC_MAX 47
#define CH_CJC_MIN 48
#define CH_CJC_MAX 59
#define CH_RES_MIN 60
#define CH_RES_MAX 61
#define CH_VREF 62
#define CH_SGND 63

#define CH_COUNT 64
#define CH_TC_COUNT 48
#define FREQ_COUNT 2
#define RANGE_COUNT 2

#define DMM_RANGE_10V 10.0
#define DMM_RANGE_100mV 0.1

#define TIMER_PERIOD (0.005)
#define OVERSAMPLE 1000
#define DISCARD 1
#define DMM_OVERSAMPLE 10

#define MAX_PAGE_READ 10

/* boundry limits: {FACT_LOW, FACT_HIGH, SELF_LOW, SELF_HIGH } */
double Zero[FREQ_COUNT][CH_TC_COUNT];
const double Zero_LIMITS[4] = { -0.015, 0.06, -0.0001, 0.0001 };
#define FE_Zero_DELAY 20

double CalZeroPos[FREQ_COUNT][CH_TC_COUNT];
const double CalZeroPos_LIMITS[4] = { -0.03, 0.07, -0.0002, 0.0002 };
#define CalZeroPos_DELAY 2

double CalZeroNeg[FREQ_COUNT][CH_TC_COUNT];
const double CalZeroNeg_LIMITS[4] = { -0.03, 0.07, -0.0002, 0.0002 };
#define CalZeroNeg_DELAY 2

double Cal65mVPos[FREQ_COUNT][CH_TC_COUNT];
const double Cal65mVPos_LIMITS[4] = { 8.72, 9.13, 0.061, 0.06325 };
#define Cal65mVPos_DELAY 10

double Cal65mVNeg[FREQ_COUNT][CH_TC_COUNT];
const double Cal65mVNeg_LIMITS[4] = { -9.09, -8.68, -0.06325, -0.061 };
#define Cal65mVNeg_DELAY 10

double CAL_65MV_DMM;
const double CAL_65MV_DMM_LIMITS[4] = { 0.0615, 0.06275, 0.0, 0.0 };
#define CAL_65MV_DMM_DELAY 2

double CAL_ZERO_DMM;
const double CAL_ZERO_DMM_LIMITS[4] = { -0.00012, 0.00012, 0.0, 0.0 };
#define CAL_ZERO_DMM_DELAY 5

double VREF_DMM;
const double VREF_DMM_LIMITS[4] = { 4.072, 4.1, 0.0, 0.0 };
#define VREF_DMM_DELAY 2

double OFFSET_DMM;
const double OFFSET_DMM_LIMITS[4] = { -0.000015, 0.000015, 0.0, 0.0 };
#define OFFSET_DMM_DELAY 2

double factSGND;
const double factSGND_LIMITS[4] = { -0.0105, 0.0105, 0.0, 0.0 };
#define factSGND_DELAY 1

double factVREF;
const double factVREF_LIMITS[4] = { 6.05, 6.2, 0.0, 0.0 };
#define factVREF_DELAY 1

FILE *logfp = NULL;

void log_array(const char *name, double v[FREQ_COUNT][CH_TC_COUNT]) {
  int i, j;

  if (logfp)
    for (i = 0; i < FREQ_COUNT; i++)
      for (j = 0; j < CH_TC_COUNT; j++)
	fprintf(logfp, "%s (freq: %i, ch: %i) = %15e\n", name, i, j, v[i][j]);
}

void log_single(const char *name, double value) {
  if (logfp)
    fprintf(logfp, "%s = %15e\n", name, value);
}

void log_error(const char *fmt, ...) {
  va_list ap;
  va_start(ap, fmt);
  vfprintf(logfp, fmt, ap);
  vfprintf(stderr, fmt, ap);
  va_end(ap);
}

void log_flush(void) {
  fflush(logfp);
  fflush(stderr);
}

/* output [chan][freq][range] */
float PosGain[RANGE_COUNT][FREQ_COUNT][CH_COUNT];
float NegGain[RANGE_COUNT][FREQ_COUNT][CH_COUNT];
float Off[RANGE_COUNT][FREQ_COUNT][CH_COUNT];
/*float cjcOff[RANGE_COUNT][FREQ_COUNT][CH_COUNT];*/
float CAL_65MV_DELTA;
float VREF;
struct dmm_device_info dmm_device_info = {"unknown", "unknown", "unknown", "unknown"};
struct dmm_cal_info dmm_cal_info = {{0, 0, 0}, {0, 0, 0}};

int factory = 0;
/*int cjc_offset = 0;*/
int self = 0;
int ignore_boundry = 0;

int scanfd = -1;
int mainfd = -1;
int fifofd = -1;

int opendev(void) {
  scanfd = open("/dev/acqscan0", O_RDWR);
  if (scanfd == -1) {
    log_error("could not open /dev/acqscan0\n");
    return -1;
  }
  mainfd = open("/dev/acqmain0", O_RDWR);
  if (mainfd == -1) {
    log_error("could not open /dev/acqmain0\n");
    return -1;
  }
  fifofd = open("/dev/acqfifo0", O_RDWR);
  if (fifofd == -1) {
    log_error("could not open /dev/acqfifo0\n");
    return -1;
  }

  logfp = fopen("/tmp/cal.log", "w");

  return 0;
}

void closedev(void) {
  if (scanfd != -1)
    close(scanfd);
  if (mainfd != -1)
    close(mainfd);
  if (fifofd != -1)
    close(fifofd);

  if (logfp)
    fclose(logfp);
}
/*static void parse_cjc_offset(const char *prgname, const char *offsets);*/

int err_flag = 1;
#define PRINTERR() if (ret && err_flag) fprintf(stderr, "ERROR: %i (%i)\n", __LINE__, ret), err_flag = 0
/* #define PRINTERR() do{}while(0) */
/* #define CHECKPT() fprintf(stderr, "CHECK: %s\n", __FUNCTION__) */
#define CHECKPT() do{}while(0)

static int factory_readings(EX1048_CLIENT *cl, struct dmm_handle *dmm);
static int calibrate(EX1048_CLIENT *clprog, EX1048_CLIENT *clcore, char *parm_str, const char *fname);
static int clear_output(void);
static int load_fact_only_vars(const char *fact_cal);
static int factory_calculations(void);
static int self_calculations(void);
static int common_calculations(void);
static int dmm_attached(struct dmm_handle *dmm);
static int write_file(const char *fname);
static int reset_system(EX1048_CLIENT *clprog, EX1048_CLIENT *clcore);
static int self_readings(EX1048_CLIENT *cl);

static void usage(const char *progname) {
  fprintf(stderr, "usage: %s [options] <output file>\n", progname);
  fprintf(stderr, "\t-t        use tcp.\n");
  fprintf(stderr, "\t-u        use udp.\n");
  fprintf(stderr, "\t-s <file> do a selfcal (give full calfile).\n");
  fprintf(stderr, "\t-f <device_id;host;gpib> full cal.\n");
  /*  fprintf(stderr, "\t-o <0;1;2;3;4;5;6;7;8;9;10;11> 12-cjc offsets.\n"); */
  fprintf(stderr, "\t-i        ignore boundry checks.\n");
  fprintf(stderr, "\t-l <lid>  specify the link id.\n");
  exit(1);
}

#define NUM_FACT_CHKPT 20
#define NUM_SELF_CHKPT 13
static void set_progress(void) {
  int percent = 0;
  FILE *progress_fp = NULL;
  int progress_fd = -1;
  static int progress = 0;

  progress_fd = open("/var/cal/progress", O_CREAT | O_WRONLY);
  if (progress_fd != -1)
    progress_fp = fdopen(progress_fd, "w");


  if (progress_fp) {
    progress++;
    if (factory)
      percent = (int)((float)progress*80 / NUM_FACT_CHKPT);
    if (self)
      percent = (int)((float)progress*80 / NUM_SELF_CHKPT);

    percent += 10; /* 10-90% in autocal */

    flock(progress_fd, LOCK_EX);  /* lock the file */
    ftruncate(progress_fd, 0); /* zero the length */
    fprintf(progress_fp, "%i%%", percent); /* write the message */
    fflush(progress_fp);
    flock(progress_fd, LOCK_UN);  /* unlock */

    fclose(progress_fp);
    close(progress_fd);
  }
}

int main(int argc, char *argv[]) {
  char optstr[] = "tuf:p:s:il:";
  char *host = "localhost";
  char *netid = "tcp";
  const char *fname;
  char *parm_str = NULL; /* factory cal file for selfcal, dmm info for factory cal */
  int c, ret=0;
  EX1048_CLIENT *clprog=NULL, *clcore=NULL;
  int lid = LID_NONE;

  while ((c = getopt(argc, argv, optstr)) != -1) {
    switch (c) {
    case 't': netid = "tcp"; break;
    case 'u': netid = "udp"; break;
    case 's': parm_str = optarg; self = 1; break;
    case 'f': parm_str = optarg; factory = 1; break;
    case 'i': ignore_boundry = 1; break;
    case 'l': lid = atoi(optarg); break;
      /*    case 'o': cjc_offset = 1; parse_cjc_offset(argv[0], optarg); break; */
    default: usage(argv[0]);
    }
  }

  if ((!factory && !self) || (factory && self))
    usage(argv[0]);

  /* output file? */
  if (optind+1 != argc)
    usage(argv[0]);
  fname = argv[optind];

  if (!ret)
    set_progress();

  if (!ret) {
    clprog = ex1048_clnt_create(host, netid, EX1048PROGVERS, lid);
    clcore = ex1048_clnt_create(host, netid, COREVERS, lid);
    ret = !clprog && !clcore;
  }

  if (!ret)
    ret = opendev();

  if (!ret)
    ret = calibrate(clprog, clcore, parm_str, fname);
  if (ret)
    log_error("calibration failed.\n");
  
  ex1048_clnt_destroy(clprog);
  ex1048_clnt_destroy(clcore);

  if (!ret)
    set_progress();

  /* write file */
  if (!ret) {

    /* reset replaces self cal file
       delete it before replacing it */
    if (self)
      unlink(fname);

    ret = write_file(fname);
    if (ret)
      log_error("could not write output file.\n");
  }

  if (!ret)
    set_progress();

  log_flush();
  closedev();

  return ret;
}

/*
 * cjc offset is not part of full cal anymore
 */

/* pull off 12 ';' separated values */
/* void parse_cjc_offset(const char *prgname, const char *offsets) { */
/*   int i; */
/*   float vals[12]; */
/*   char *end; */

/*   for (i = 0; i < 12; i++) { */
/*     vals[i] = strtof(offsets, &end); */
/*     if (*end != ';' && *end != '\0') */
/*       usage(prgname); */
/*     offsets = end+1; */
/*   } */
/*   if (i != 12) */
/*     usage(prgname); */

/*   /\* have 12 values *\/ */
/*   for (i = 0; i < CH_COUNT; i++) { */
/*     /\* cjc *\/ */
/*     if (i >= CH_CJC_MIN && i <= CH_CJC_MAX) { */
/*       cjcOff[0][0][i] = vals[i-CH_CJC_MIN]; */
/*       cjcOff[0][1][i] = vals[i-CH_CJC_MIN]; */
/*       cjcOff[1][0][i] = vals[i-CH_CJC_MIN]; */
/*       cjcOff[1][1][i] = vals[i-CH_CJC_MIN]; */
/*     } */
/*     /\* default *\/ */
/*     else { */
/*       cjcOff[0][0][i] = 0.0; */
/*       cjcOff[0][1][i] = 0.0; */
/*       cjcOff[1][0][i] = 0.0; */
/*       cjcOff[1][1][i] = 0.0; */
/*     } */
/*   } */
/* } */

static int write_file(const char *fname) {
  struct calfile cal;

  if (cal_open(&cal, fname, CAL_CREATE) != CAL_ERR_NONE)
    return 1;

  cal_setvalue(&cal, "vender", "VXI Technology, Inc.");
  cal_setvalue(&cal, "device", "EX1048");
  cal_setvalue(&cal, "rev", "1");

  if (factory) {
    char tmp[128];

    cal_setvalue(&cal, "dmm_manufacturer", dmm_device_info.manufacturer);
    cal_setvalue(&cal, "dmm_model", dmm_device_info.model);
    cal_setvalue(&cal, "dmm_serial", dmm_device_info.serial);
    cal_setvalue(&cal, "dmm_version", dmm_device_info.version);

    sprintf(tmp, "%02i/%02i/%04i", dmm_cal_info.date.month, dmm_cal_info.date.day, dmm_cal_info.date.year);
    cal_setvalue(&cal, "dmm_cal_date", tmp);

    sprintf(tmp, "%02i/%02i/%04i", dmm_cal_info.due.month, dmm_cal_info.due.day, dmm_cal_info.due.year);
    cal_setvalue(&cal, "dmm_cal_due", tmp);

    cal_adddata(&cal, &CAL_65MV_DELTA, CALTYPE_CAL_65MV_DELTA);
    cal_adddata(&cal, &VREF, CALTYPE_VREF);
    cal_adddata(&cal, PosGain, CALTYPE_FACT_GAIN_POS);
    cal_adddata(&cal, NegGain, CALTYPE_FACT_GAIN_NEG);
    cal_adddata(&cal, Off, CALTYPE_FACT_OFFSET);
    /*    if (cjc_offset)
	  cal_adddata(&cal, cjcOff, CALTYPE_USER_OFFSET);*/
  }
  else if (self) {
    cal_adddata(&cal, PosGain, CALTYPE_SELF_GAIN_POS);
    cal_adddata(&cal, NegGain, CALTYPE_SELF_GAIN_NEG);
    cal_adddata(&cal, Off, CALTYPE_SELF_OFFSET);
  }
  
  if (cal_close(&cal) != CAL_ERR_NONE)
    return 1;

  return 0;
}

static int set_cal_mode(int flag) {
  flag = !!flag;
  return ioctl(scanfd, SCAN_CAL_MODE_SET, &flag);
}

static int set_led_update(int flag) {
  flag = !!flag;
  return ioctl(scanfd, SCAN_LED_UPDATE_SET, &flag);
}

static int calibrate(EX1048_CLIENT *clprog, EX1048_CLIENT *clcore, char *parm_str, const char *fname) {
  int ret = 0;
  EX1048_RPCresult result;

  CHECKPT();

  /* massive system reset */
  ret = clear_output();
  if (!ret)
    ret = reset_0x00000101(clprog->lid, &result, clprog->rpc_cl);
  if (!ret)
    ret = result.error_code;
  if (ret)
    log_error("system reset failed\n");
  if (!ret)
    ret = system("loadcal -d");
  if (ret)
    log_error("could not load default cal data!\n");

  if (!ret)
    set_progress();

  if (!ret)
    ret = set_led_update(0);
  if (!ret)
    ret = set_cal_mode(1);

  if (factory) {
    /* DO A FACTORY CAL */
    struct dmm_params params;
    struct dmm_handle *dmm=NULL;
    char host[256];
    char gpib_addr[256];
    params.host = host;
    params.gpib_addr = gpib_addr;
    printf("performing factory cal.\n");

    if (!ret) {
      ret = (sscanf(parm_str, "%i;%[^;];%[^;]",
		    &params.device, host, gpib_addr) != 3);
    }
    if (!ret) {
      dmm = dmm_volt_init(&params);
      if ((ret = !dmm))
	log_error("could not connect to the dmm\n");
    }

    if (!ret)
      set_progress();

    /* attached to DMM? */
    if (!ret)
      ret = dmm_attached(dmm);

    /* do factory readings */
    if (!ret)
      ret = factory_readings(clprog, dmm);

    if (dmm)
      ret = dmm_destroy(dmm) || ret;
  }
  else {
    /* DO A SELF CAL */
    char *tmp = malloc(strlen(parm_str) + 64);

    printf("performing self cal.\n");

    if (!ret) {
      sprintf(tmp, "loadcal %s", parm_str);
      ret = system(tmp);
      if (ret)
	log_error("could not load factory cal (%s)\n", parm_str);
    }

    if (!ret)
      ret = self_readings(clprog);
  
    /* load CAL_65MV_DELTA & VREF from factory cal file */
    if (!ret) {
      ret = load_fact_only_vars(parm_str);
      if (ret)
	log_error("could not load necessary variables from the factory cal file.\n");
    }

    free(tmp);
  }

  if (!ret)
    set_progress();

  if (!ret && factory)
    ret = factory_calculations();
  else if (!ret && self)
    ret = self_calculations();

  if (!ret)
    ret = common_calculations();

  if (!ret)
    ret = reset_system(clprog, clcore);

  if (!ret)
    set_progress();

  if (!ret)
    printf("cal readings/calculations successful!\n");

  return ret;
}

static int load_fact_only_vars(const char *fact_cal) {
  struct calfile cal;
  uint32_t type;
  int found_cal_65mv_delta = 0, found_vref = 0;

  CHECKPT();

  if (cal_open(&cal, fact_cal, CAL_READ) != CAL_ERR_NONE)
    return 1;

  if (cal_resetdataenum(&cal) != CAL_ERR_NONE)
    return 1;

  while (1) {
    if (cal_getnextdatatype(&cal, &type) != CAL_ERR_NONE)
      return 1;

    if (!type)
      break;

    switch(type) {
    case CALTYPE_CAL_65MV_DELTA:
      if (cal_getdata(&cal, (uint32_t*)&CAL_65MV_DELTA) != CAL_ERR_NONE)
	return 1;
      found_cal_65mv_delta = 1;
      break;
    case CALTYPE_VREF:
      if (cal_getdata(&cal, (uint32_t*)&VREF) != CAL_ERR_NONE)
	return 1;
      found_vref = 1;
      break;
    default:
      /* skip all others */
      if (cal_getdata(&cal, NULL) != CAL_ERR_NONE)
	return 1;
      break;
    }
  }

  if (cal_close(&cal) != CAL_ERR_NONE)
    return 1;

  return !(found_cal_65mv_delta && found_vref);
}

/* static int enable_channels(CLIENT *cl, int on) {   */
/*   EX1048_RPCresult result; */
/*   int i; */
/*   int count = on?48:0; */
/*   int channels[48]; */
/*   int ret; */

/*   CHECKPT(); */

/*   for (i = 0; i < count; i++) */
/*     channels[i] = i; */

/*   ret = setscanlist_cmd(cl, EX1048_CURRENT_SCANLIST, count, channels, &result); */
  
/*   if (ret) */
/*     return ret; */
/*   return result.error_code; */
/* } */

static int enable_channels(EX1048_CLIENT *cl, int on) {
  struct acqengine_scanlist scanlist;
  int count = on?48:0;
  int i;

  CHECKPT();

  /* enable all 48 channels */
  bzero(&scanlist, sizeof(scanlist));
  scanlist.num = count;
  for (i = 0; i < count; i++)
    scanlist.channels[i] = i;

  i = ioctl(scanfd, SCAN_LIST_SET_USER, &scanlist);

  return i;
}


static int set_freq(EX1048_CLIENT *cl, int freq_hi) {
  int channels[48];
  double freqs[48];
  EX1048_RPCresult result;
  int ret, i;

  CHECKPT();

  for (i = 0; i < 48; i++) {
    channels[i] = i;
    freqs[i] = freq_hi?1000.0:4.0;
  }

  ret = setfreqs_cmd(cl, 48, channels, freqs, &result);
  if (ret)
    return ret;
  return result.error_code;
}

/* static int set_freq(CLIENT *cl, int freq_hi) { */
/*   EX1048_RPCresult result; */
/*   EX1048_RPCFilterList list; */
/*   int i; */
/*   int channels[48]; */
/*   double cutoff[48]; */
/*   double freq; */

/*   CHECKPT(); */

/*   freq = freq_hi ? 1000.0 : 4.0; */

/*   bzero(&list, sizeof(list)); */
/*   for (i = 0; i < 48; i++) { */
/*     channels[i] = i; */
/*     cutoff[i] = freq; */
/*   } */
  
/*   list.channels.channels_len = 48; */
/*   list.cutoff_freq.cutoff_freq_len = 48; */
/*   list.channels.channels_val = channels; */
/*   list.cutoff_freq.cutoff_freq_val = cutoff; */
  
/*   if (set_filt_freq_1(list, &result, cl) != RPC_SUCCESS) */
/*     return 1; */

/*   return result.error_code; */
/* } */

/* !!!! does this irreversibly screw things up? */
int set_calmux(uint32_t mask) {
  int ret;

  CHECKPT();

  ret = ioctl(scanfd, SCAN_MUX_SET, &mask);

  return ret;
}

static int configure_trig(EX1048_CLIENT *cl) {
  EX1048_RPCresult result;
  EX1048_RPCTrigSrc trigsrc;

  CHECKPT();

  /* set arm source to imm */
  bzero(&trigsrc, sizeof(trigsrc));
  trigsrc.imm = 1;
  
  /* set the LID */
  trigsrc.lid = cl->lid;
  
  if (trig_arm_source_set_0x00000101(trigsrc, &result, cl->rpc_cl) != RPC_SUCCESS ||
      result.error_code != EX1048_SUCCESS)
    return 1;

  /* set trig source to timer */
  bzero(&trigsrc, sizeof(trigsrc));
  trigsrc.timer_enable = 1;
  
  /* set the LID */
  trigsrc.lid = cl->lid;
  
  if (trig_trig_source_set_0x00000101(trigsrc, &result, cl->rpc_cl) != RPC_SUCCESS ||
      result.error_code != EX1048_SUCCESS)
    return 1;

  return 0;
}

static int reset_trig(EX1048_CLIENT *cl) {
  EX1048_RPCresult result;
  EX1048_RPCTrigSrc trigsrc;

  CHECKPT();

  /* set arm source to imm */
  bzero(&trigsrc, sizeof(trigsrc));
  trigsrc.imm = 1;

  /* set the LID */
  trigsrc.lid = cl->lid;
  
  if (trig_arm_source_set_0x00000101(trigsrc, &result, cl->rpc_cl) != RPC_SUCCESS ||
      result.error_code != EX1048_SUCCESS)
    return 1;

  /* set trig source to imm */
  bzero(&trigsrc, sizeof(trigsrc));
  trigsrc.imm = 1;
  
  /* set the LID */
  trigsrc.lid = cl->lid;
  
  if (trig_trig_source_set_0x00000101(trigsrc, &result, cl->rpc_cl) != RPC_SUCCESS ||
      result.error_code != EX1048_SUCCESS)
    return 1;

  return 0;
}

static int get_readings(EX1048_CLIENT *cl, double *ch_out, int channels, int pages) {
  EX1048_RPCFIFOArray rpc_data = { 0, 0, { 0, NULL} };
  EX1048_RPCSampleData_p data;
  EX1048_RPCresult result;
  EX1048_RPCTrigCount count;
  EX1048_RPCFIFOPageCount page_count;
  int i, j;
  int received = 0;
  int ret = 0;

  CHECKPT();

  if (pages < 1)
    pages = 1;
  else if (pages > 1000)
    pages = 1000;

  count.count = pages;
  
  /* set the LID */
  count.lid = cl->lid;
  
  if (trig_trig_count_set_0x00000101(count, &result, cl->rpc_cl) != RPC_SUCCESS ||
      result.error_code != EX1048_SUCCESS)
    return 1;

  if (trig_init_0x00000101(cl->lid, &result, cl->rpc_cl) != RPC_SUCCESS ||
      result.error_code != EX1048_SUCCESS)
    return 1;

  for (i = 0; i < channels; i++)
    ch_out[i] = 0.0;
    
  while (received < pages) {
    memset(&page_count, 0, sizeof(page_count));
    page_count.lid = cl->lid;
    page_count.page_count = pages > MAX_PAGE_READ ? MAX_PAGE_READ : pages;
    if (read_fifo_0x00000101(page_count, &rpc_data, cl->rpc_cl) != RPC_SUCCESS ||
	result.error_code != EX1048_SUCCESS)
      return 1;

    /* no pages? */
    if (rpc_data.fifo.fifo_len < 1) {
      sleep(1);
      continue;
    }
    
    for (i = 0; i < rpc_data.fifo.fifo_len; i++) {
      data = rpc_data.fifo.fifo_val[i];
      
      /* not what we expect? */
      if (data->x.x_len != channels) {
	ret = 1;
	break;
      }
      
      for (j = 0; j < channels; j++)
	ch_out[j] += data->x.x_val[j];
    }
    
    received += rpc_data.fifo.fifo_len;
    xdr_free( (xdrproc_t) xdr_EX1048_RPCFIFOArray, (char *) &rpc_data);
  }

  for (i = 0; i < channels; i++)
    ch_out[i] /= (double)pages;

  return ret;
}

static int set_timer(EX1048_CLIENT *cl, double period) {
  EX1048_RPCTrigTimer timer;
  EX1048_RPCresult result;

  CHECKPT();

  timer.timer_period = period;
  
  /* set the LID */
  timer.lid = cl->lid;
  
  if (trig_timer_period_set_0x00000101(timer, &result, cl->rpc_cl) != RPC_SUCCESS)
    return 1;

  return result.error_code;
}

static int read_both_freq(EX1048_CLIENT *cl, double data[][CH_TC_COUNT], int channels, int oversample) {
  int ret = 0;

  CHECKPT();

  if (!ret) /* low freq */
    ret = set_freq(cl, 0);
  if (!ret) /* get the readings */
    ret = get_readings(cl, data[0], channels, oversample);
  if (!ret) /* high freq */
    ret = set_freq(cl, 1);
  if (!ret) /* get the readings */
    ret = get_readings(cl, data[1], channels, oversample);

  return ret;
}

static int enable_cjc_channel(EX1048_CLIENT *cl, int ch) {
  struct scan_cjc_list list;
  int ret = 0;

  CHECKPT();

  list.count = 1;
  list.list[0] = ch;

  ret = ioctl(scanfd, SCAN_CJC_CHANNELS_SET, &list);

  return ret;
}

static int reset_cjc_channels(EX1048_CLIENT *cl) {
  struct scan_cjc_list list;
  int ret = 0, i;
  int count = CH_CJC_MAX-CH_CJC_MIN+1;

  CHECKPT();

  list.count = count;
  for (i = 0; i < count; i++)
    list.list[i] = CH_CJC_MIN + i;

  ret = ioctl(scanfd, SCAN_CJC_CHANNELS_SET, &list);

  return ret;
}

static int report_cjc(EX1048_CLIENT *cl, int enable) {
  EX1048_RPCFIFOFormat in, out;

  CHECKPT();

  bzero(&in, sizeof(in));
  in.report_cjc = !!enable;
  in.report_timestamp = 1;
  in.report_count = 1;
  
  /* set the LID */
  in.lid = cl->lid;
  
  if (set_fifo_config_0x00000101(in, &out, cl->rpc_cl) != RPC_SUCCESS)
    return 1;

  return out.error_code;
}


static int set_conv_mV(EX1048_CLIENT *cl) {
  EX1048_RPCresult result;
  EX1048_RPCEUConversion conversions[48];
  int channels[48];
  int ret, i;

  for (i = 0; i < 48; i++) {
    channels[i] = i;
    conversions[i] = EX1048_RPC_EU_MV;
  }

  ret = setconversions_cmd(cl, 48, channels, conversions, &result);
  if (ret)
    return ret;
  return result.error_code;
}

#define CONV_mV 0
#define CONV_CJC 13
/* static int set_conv_mV(void) { */
/*   struct acqengine_conversions list; */
/*   int i; */

/*   CHECKPT(); */
  
/*   for (i = 0; i < sizeof(list.conversions)/sizeof(__u32); i++) */
/*     list.conversions[i] = CONV_mV; */
  
/*   return ioctl(scanfd, SCAN_CONV_SET, &list); */
/* } */

static int enable_fact_mode(int enable) {
  enable = !!enable;

  CHECKPT();

  /* enable factory mode */
  return ioctl(mainfd, ENGINE_SET_FACT_MODE, &enable);
}

static int cjc_conv_mV(int enable) {
  int ret = 0;
  __u32 cjc_conv;

  CHECKPT();

  /* set cjc conv */
  if (!ret) {
    cjc_conv = enable?CONV_mV:CONV_CJC;
    ret = ioctl(scanfd, SCAN_CJC_CONV_SET, &cjc_conv);
  }

  return ret;
}

static int read_cjc(EX1048_CLIENT *cl, double *out, int oversample, int discard) {
  EX1048_RPCFIFOArray rpc_data = { 0, 0, { 0, NULL} };
  EX1048_RPCSampleData_p data;
  EX1048_RPCresult result;
  EX1048_RPCTrigCount count;
  EX1048_RPCFIFOPageCount page_count;
  int i;
  int received = 0;
  int ret = 0;
  int discard_save = discard;

  CHECKPT();

  if (oversample < 1)
    oversample = 1;
  else if (oversample > 1000)
    oversample = 1000;

  count.count = oversample;
  
  /* set the LID */
  count.lid = cl->lid;
  
  if (trig_trig_count_set_0x00000101(count, &result, cl->rpc_cl) != RPC_SUCCESS ||
      result.error_code != EX1048_SUCCESS)
    return 1;

  if (trig_init_0x00000101(cl->lid, &result, cl->rpc_cl) != RPC_SUCCESS ||
      result.error_code != EX1048_SUCCESS)
    return 1;

  *out = 0.0;
    
  while (received < oversample) {
    memset(&page_count, 0, sizeof(page_count));
    page_count.lid = cl->lid;
    page_count.page_count = oversample > MAX_PAGE_READ ? MAX_PAGE_READ : oversample;
    if (read_fifo_0x00000101(page_count, &rpc_data, cl->rpc_cl) != RPC_SUCCESS ||
	result.error_code != EX1048_SUCCESS)
      return 1;

    /* no pages? */
    if (rpc_data.fifo.fifo_len < 1) {
      sleep(1);
      continue;
    }
    
    for (i = 0; i < rpc_data.fifo.fifo_len; i++) {
      /* discard the first N samples */
      if (discard > 0) {
	discard--;
	continue;
      }

      data = rpc_data.fifo.fifo_val[i];
      
      /* not what we expect? */
      if (data->cjc.cjc_len != 1) {
	ret = 1;
	break;
      }
      
      *out += data->cjc.cjc_val[0];
    }
    
    received += rpc_data.fifo.fifo_len;
    xdr_free( (xdrproc_t) xdr_EX1048_RPCFIFOArray, (char *) &rpc_data);
  }

  *out /= (double)(oversample-discard_save);

  return ret;
}

static int disable_mux_discharge(int disable) {
  int ret = 0;

  CHECKPT();

  ret = ioctl(fifofd, FIFO_MUX_DISCHARGE, &disable);

  return ret;
}

#define sanitycheck_single(name) __sanitycheck_single(#name, name, name##_LIMITS)
static int __sanitycheck_single(const char *field, double val, const double limits[4]) {
  int ret = 0;
  double low =  limits[(!!self)*2 + 0];
  double high = limits[(!!self)*2 + 1];

  ret =  val > high || val < low;
  if (ret)
    log_error("%s = %15.15e outside bounds: [%f, %f]\n", field, val, low, high);
  return ret && !ignore_boundry;
}

#define sanitycheck_array(name) __sanitycheck_array(#name, name, name##_LIMITS)
static int __sanitycheck_array(const char *field, double val[FREQ_COUNT][CH_TC_COUNT],
			       const double limits[4]) {
  int i, j;
  int ret = 0;
  double low =  limits[(!!self)*2 + 0];
  double high = limits[(!!self)*2 + 1];

  for (i = 0; i < FREQ_COUNT; i++)
    for (j = 0; j < CH_TC_COUNT; j++)
      if (val[i][j] > high || val[i][j] < low) {
	ret = 1;
	log_error("%s[freq: %i, ch: %i] = %15.15e outside bounds: [%f, %f]\n",
		  field, i, j, val[i][j], low, high);
      }
  return ret && !ignore_boundry;
}

/* cal stuff */
static int measure_dmm(struct dmm_handle *dmm, double *out) {
  int i, ret=0;
  double tmp;
  
  /*return dmm_read(dmm,out);*/

  *out = 0.0;
  for (i = 0; (!ret) && (i < DMM_OVERSAMPLE); i++) {
    ret = dmm_read(dmm, &tmp);
    /*printf("measure_dmm: %i %f\n", ret, tmp);*/
    *out += tmp;
  }

  *out = (*out) / (float)DMM_OVERSAMPLE;
  return ret;
}

static int measure_FE_Zero(EX1048_CLIENT *cl) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    ret = enable_channels(cl, 1);
  if (!ret)
    ret = set_conv_mV(cl);
  PRINTERR();
  if (!ret)
    ret = configure_trig(cl);
  PRINTERR();
  if (!ret)
    ret = set_timer(cl, TIMER_PERIOD); /* 200 Hz */
  PRINTERR();
  if (!ret) /* set ADC to read FE zeros */
    ret = set_calmux(CALMUX_CAL_MUX1 | CALMUX_CAL_MUX2 | CALMUX_CAL_MUX3
		     | CALMUX_CAL_MUX4 | CALMUX_CAL_65MV | CALMUX_CAL_45MV);
  PRINTERR();
  sleep(FE_Zero_DELAY);
  if (!ret)
    ret = read_both_freq(cl, Zero, 48, OVERSAMPLE);
  PRINTERR();
  if (!ret)
    log_array("Zero", Zero);
  else
    log_error("Error acquiring FE Zero\n");
  if (!ret)
    ret = sanitycheck_array(Zero);
  return ret;
}

static int measure_VREF_DMM(struct dmm_handle *dmm) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    ret = dmm_set_volt_range(dmm, DMM_RANGE_10V);
  if (!ret)
    ret = set_calmux(CALMUX_CAL_MUX1 | CALMUX_CAL_MUX2 | CALMUX_CAL_MUX3
		     | CALMUX_CAL_MUX4 | CALMUX_CAL_65MV | CALMUX_CAL_45MV
		     | CALMUX_VREF_DMM | CALMUX_GND_DMM);
  sleep(VREF_DMM_DELAY);
  if (!ret)
    measure_dmm(dmm, &VREF_DMM);
  if (!ret)
    log_single("VREF_DMM", VREF_DMM);
  else
    log_error("Error acquiring VREF_DMM\n");
  if (!ret)
    ret = sanitycheck_single(VREF_DMM);
  return ret;
}

static int measure_OFFSET_DMM(struct dmm_handle *dmm) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    ret = dmm_set_volt_range(dmm, DMM_RANGE_100mV);
  if (!ret)
    ret = set_calmux(CALMUX_CAL_MUX1 | CALMUX_CAL_MUX2 | CALMUX_CAL_MUX3
		     | CALMUX_CAL_MUX4 | CALMUX_CAL_65MV | CALMUX_CAL_45MV
		     | CALMUX_SHORT_DMM);
  sleep(OFFSET_DMM_DELAY);
  if (!ret)
    ret = measure_dmm(dmm, &OFFSET_DMM);
  if (!ret)
    log_single("OFFSET_DMM", OFFSET_DMM);
  else
    log_error("Error acquiring OFFSET_DMM\n");
  if (!ret)
    ret = sanitycheck_single(OFFSET_DMM);
  return ret;
}

static int measure_CAL_ZERO_DMM(struct dmm_handle *dmm) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    ret = dmm_set_volt_range(dmm, DMM_RANGE_100mV);
  if (!ret)
    ret = set_calmux(CALMUX_CAL_MUX2 | CALMUX_CAL_MUX4 | CALMUX_CAL_65MV
		     | CALMUX_CAL_45MV | CALMUX_VCAL_DMM | CALMUX_GND_DMM);
  sleep(CAL_ZERO_DMM_DELAY);
  if (!ret)
    ret = measure_dmm(dmm, &CAL_ZERO_DMM);
  if (!ret)
    log_single("CAL_ZERO_DMM", CAL_ZERO_DMM);
  else
    log_error("Error acquiring CAL_ZERO_DMM\n");
  if (!ret)
    ret = sanitycheck_single(CAL_ZERO_DMM);
  return ret;
}

static int measure_CalZeroPos(EX1048_CLIENT *cl) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    ret = enable_channels(cl, 1);
  if (!ret)
    ret = set_conv_mV(cl);
  if (!ret)
    ret = configure_trig(cl);
  if (!ret)
    ret = set_timer(cl, TIMER_PERIOD); /* 200 Hz */
  if (!ret)
    ret = set_calmux(CALMUX_CAL_MUX2 | CALMUX_CAL_MUX4 | CALMUX_CAL_65MV
		     | CALMUX_CAL_45MV | CALMUX_VCAL_DMM | CALMUX_GND_DMM);
  sleep(CalZeroPos_DELAY);
  if (!ret)
    ret = read_both_freq(cl, CalZeroPos, 48, OVERSAMPLE);
  if (!ret)
    log_array("CalZeroPos", CalZeroPos);
  else
    log_error("Error acquiring CalZeroPos\n");
  if (!ret)
    ret = sanitycheck_array(CalZeroPos);
  return ret;
}

static int measure_CalZeroNeg(EX1048_CLIENT *cl) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    ret = enable_channels(cl, 1);
  if (!ret)
    ret = set_conv_mV(cl);
  if (!ret)
    ret = configure_trig(cl);
  if (!ret)
    ret = set_timer(cl, TIMER_PERIOD); /* 200 Hz */
  if (!ret)
    ret = set_calmux(CALMUX_CAL_MUX1 | CALMUX_CAL_MUX3 | CALMUX_CAL_65MV
		     | CALMUX_CAL_45MV | CALMUX_VCAL_DMM | CALMUX_GND_DMM);
  sleep(CalZeroNeg_DELAY);
  if (!ret)
    ret = read_both_freq(cl, CalZeroNeg, 48, OVERSAMPLE);
  if (!ret)
    log_array("CalZeroNeg", CalZeroNeg);
  else
    log_error("Error acquiring CalZeroNeg\n");
  if (!ret)
    ret = sanitycheck_array(CalZeroNeg);
  return ret;
}

static int measure_CAL_65MV_DMM(struct dmm_handle *dmm) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    ret = dmm_set_volt_range(dmm, DMM_RANGE_100mV);
  if (!ret)
    ret = set_calmux(CALMUX_CAL_MUX2 | CALMUX_CAL_MUX4 | CALMUX_CAL_45MV
		     | CALMUX_CAL_ZERO | CALMUX_VCAL_DMM | CALMUX_GND_DMM);
  sleep(CAL_65MV_DMM_DELAY);
  if (!ret)
    ret = measure_dmm(dmm, &CAL_65MV_DMM);
  if (!ret)
    log_single("CAL_65MV_DMM", CAL_65MV_DMM);
  else
    log_error("Error acquiring CAL_65MV_DMM\n");
  if (!ret)
    ret = sanitycheck_single(CAL_65MV_DMM);
  return ret;
}

static int measure_Cal65mVPos(EX1048_CLIENT *cl) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    ret = enable_channels(cl, 1);
  if (!ret)
    ret = set_conv_mV(cl);
  if (!ret)
    ret = configure_trig(cl);
  if (!ret)
    ret = set_timer(cl, TIMER_PERIOD); /* 200 Hz */
  if (!ret)
    ret = set_calmux(CALMUX_CAL_MUX2 | CALMUX_CAL_MUX4 | CALMUX_CAL_45MV
    | CALMUX_CAL_ZERO | CALMUX_VCAL_DMM | CALMUX_GND_DMM);
  sleep(Cal65mVPos_DELAY);
  if (!ret)
    ret = read_both_freq(cl, Cal65mVPos, 48, OVERSAMPLE);
  if (!ret)
    log_array("Cal65mVPos", Cal65mVPos);
  else
    log_error("Error acquiring Cal65mVPos\n");
  if (!ret)
    ret = sanitycheck_array(Cal65mVPos);
  return ret;
}

static int measure_Cal65mVNeg(EX1048_CLIENT *cl) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    ret = enable_channels(cl, 1);
  if (!ret)
    ret = set_conv_mV(cl);
  if (!ret)
    ret = configure_trig(cl);
  if (!ret)
    ret = set_timer(cl, TIMER_PERIOD); /* 200 Hz */
  if (!ret)
    ret = set_calmux(CALMUX_CAL_MUX1 | CALMUX_CAL_MUX3 | CALMUX_CAL_45MV
    | CALMUX_CAL_ZERO | CALMUX_VCAL_DMM | CALMUX_GND_DMM);
  sleep(Cal65mVNeg_DELAY);
  if (!ret)
    ret = read_both_freq(cl, Cal65mVNeg, 48, OVERSAMPLE);
  if (!ret)
    log_array("Cal65mVNeg", Cal65mVNeg);
  else
    log_error("Error acquiring Cal65mVNeg\n");
  if (!ret)
    ret = sanitycheck_array(Cal65mVNeg);
  return ret;
}

static int measure_factSGND(EX1048_CLIENT *cl) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    ret = set_calmux(CALMUX_CAL_MUX1 | CALMUX_CAL_MUX2 | CALMUX_CAL_MUX3
		     | CALMUX_CAL_MUX4 | CALMUX_CAL_65MV | CALMUX_CAL_45MV);
  if (!ret)
    ret = report_cjc(cl, 1);
  if (!ret)
    ret = enable_fact_mode(1);
  if (!ret)
    ret = cjc_conv_mV(1);
  if (!ret)
    ret = set_freq(cl, 0);
  if (!ret)
    ret = configure_trig(cl);
  if (!ret)
    ret = set_timer(cl, TIMER_PERIOD);
  if (!ret)
    ret = disable_mux_discharge(1);
  if (!ret)
    ret = enable_cjc_channel(cl, CH_SGND);
  if (!ret)
    ret = enable_channels(cl, 0);
  sleep(factSGND_DELAY);
  if (!ret)
    ret = read_cjc(cl, &factSGND, OVERSAMPLE, DISCARD);
  if (!ret)
    log_single("factSGND", factSGND);  
  else
    log_error("Error acquiring factSGND\n");
  if (!ret)
    ret = sanitycheck_single(factSGND);
  if (!ret)
    ret = disable_mux_discharge(0);
  if (!ret)
    ret = cjc_conv_mV(0);
  if (!ret)
    ret = enable_fact_mode(0);
  if (!ret)
    ret = report_cjc(cl, 0);
  return ret;
}

static int measure_factVREF(EX1048_CLIENT *cl) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    ret = set_calmux(CALMUX_CAL_MUX1 | CALMUX_CAL_MUX2 | CALMUX_CAL_MUX3
		     | CALMUX_CAL_MUX4 | CALMUX_CAL_65MV | CALMUX_CAL_45MV);
  if (!ret)
    ret = report_cjc(cl, 1);
  if (!ret)
    ret = enable_fact_mode(1);
  if (!ret)
    ret = cjc_conv_mV(1);
  if (!ret)
    ret = set_freq(cl, 0);
  if (!ret)
    ret = configure_trig(cl);
  if (!ret)
    ret = set_timer(cl, TIMER_PERIOD);
  if (!ret)
    ret = disable_mux_discharge(1);
  if (!ret)
    ret = enable_cjc_channel(cl, CH_VREF);
  if (!ret)
    ret = enable_channels(cl, 0);
  sleep(factVREF_DELAY);
  if (!ret)
    ret = read_cjc(cl, &factVREF, OVERSAMPLE, DISCARD);
  if (!ret)
    log_single("factVREF", factVREF);  
  else
    log_error("Error acquiring factVREF\n");
  if (!ret)
    ret = sanitycheck_single(factVREF);
  if (!ret)
    ret = disable_mux_discharge(0);
  if (!ret)
    ret = cjc_conv_mV(0);
  if (!ret)
    ret = enable_fact_mode(0);
  if (!ret)
    ret = report_cjc(cl, 0);
  return ret;
}

static int self_readings(EX1048_CLIENT *cl) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_FE_Zero(cl);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_CalZeroPos(cl);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_CalZeroNeg(cl);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_Cal65mVPos(cl);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_Cal65mVNeg(cl);
  if (!ret)
    set_progress();
  if (ret)
    log_error("Self Readings Failed.\n");
  if (!ret)
    set_progress();
  return ret;
}

static int factory_readings(EX1048_CLIENT *cl, struct dmm_handle *dmm) {
  int ret = 0;

  CHECKPT();

  if (!ret)
    ret = dmm_get_device_info(dmm, &dmm_device_info);
  if (!ret)
    log_error("get_device_info was successful\n");
    ret = dmm_get_cal_info(dmm, &dmm_cal_info);
  if (ret)
    log_error("Error acquiring dmm info\n");

  if (!ret)
    set_progress();
  /* normal calibration steps */
  if (!ret)
    ret = measure_FE_Zero(cl);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_OFFSET_DMM(dmm);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_CAL_ZERO_DMM(dmm);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_CalZeroPos(cl);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_CalZeroNeg(cl);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_CAL_65MV_DMM(dmm);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_Cal65mVPos(cl);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_Cal65mVNeg(cl);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_VREF_DMM(dmm);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_factSGND(cl);
  if (!ret)
    set_progress();
  if (!ret)
    ret = measure_factVREF(cl);
  if (!ret)
    set_progress();
  if (ret)
    log_error("Factory Readings Failed.\n");
  if (!ret)
    set_progress();
  return ret;
}

static int clear_output(void) {
  int i, j, k;

  CHECKPT();

  for (i = 0; i < RANGE_COUNT; i++)
    for (j = 0; j < FREQ_COUNT; j++)
      for (k = 0; k < CH_COUNT; k++) {
	PosGain[i][j][k] = 1.0f;
	NegGain[i][j][k] = 1.0f;
	Off[i][j][k] = 0.0f;
      }

  return 0;
}

static int self_calculations(void) {
  int i;

  CHECKPT();

  /* CJC, Pos, 4Hz -- 1.0 */
  for (i = CH_CJC_MIN; i <= CH_CJC_MAX; i++)
    PosGain[0][0][i] = 1.0f;
  
  /* CJC, 4Hz -- 0.0 */
  for (i = CH_CJC_MIN; i <= CH_CJC_MAX; i++)
    Off[0][0][i] = 0.0f;

  return 0;
}

static int factory_calculations(void) {
  int i;

  CHECKPT();
  
  CAL_65MV_DELTA = (float)(CAL_65MV_DMM - CAL_ZERO_DMM);

  VREF = (float)VREF_DMM;

  /* CJC, 4Hz */
  for (i = CH_CJC_MIN; i <= CH_CJC_MAX; i++)
    Off[0][0][i] = (float)factSGND;
  
  /* CJC, Pos, 4Hz */
  for (i = CH_CJC_MIN; i <= CH_CJC_MAX; i++)
    PosGain[0][0][i] = (float)(VREF_DMM / (factVREF - factSGND));

  return 0;
}

static int common_calculations(void) {
  int i, j;

  CHECKPT();

  /* TC, 1kHz/4Hz */
  for (i = CH_TC_MIN; i <= CH_TC_MAX; i++)
    for (j = 0; j < FREQ_COUNT; j++)
      Off[0][j][i] = (float)Zero[j][i];

  /* TC, Pos, 1kHz/4Hz */
  for (i = CH_TC_MIN; i <= CH_TC_MAX; i++)
    for (j = 0; j < FREQ_COUNT; j++) {
      PosGain[0][j][i] = (float)(CAL_65MV_DELTA / (Cal65mVPos[j][i] - CalZeroPos[j][i]));
      NegGain[0][j][i] = (float)(-CAL_65MV_DELTA / (Cal65mVNeg[j][i] - CalZeroNeg[j][i]));
    }

  return 0;
}

/* if we can't measure VREF & GND -- then assume we are not attached */
static int dmm_attached(struct dmm_handle *dmm) {
  double tmp;
  int ret = 0;
  int count=4;

  CHECKPT();

/*   printf("checking if dmm is attached...\n"); */

  /* switch between VREF and GND a few times... */
  while (!ret && count) {
    if (count % 2) {
      if (!ret)
	ret = dmm_set_volt_range(dmm, DMM_RANGE_10V);
      PRINTERR();
      if (!ret)
	ret = set_calmux(CALMUX_CAL_MUX1 | CALMUX_CAL_MUX2 | CALMUX_CAL_MUX3
			 | CALMUX_CAL_MUX4 | CALMUX_CAL_65MV | CALMUX_CAL_45MV
			 | CALMUX_VREF_DMM | CALMUX_GND_DMM);
      PRINTERR();
      sleep(2);
      if (!ret)
	ret = dmm_read(dmm, &tmp);
      PRINTERR();
      if (ret) {
	log_error("Error testing if dmm is attached!\n");
	logerror(VXILOG_CRIT, "Error finding dmm");
	return 1;
      }
      else {
/* 	printf("testing VREF\n"); */
	ret = !(tmp > 4.05 && tmp < 4.1);
	if (ret)
	  break;
      }
      PRINTERR();
    }
    else {
      if (!ret)
	ret = dmm_set_volt_range(dmm, DMM_RANGE_100mV);
      PRINTERR();
      if (!ret)
	ret = set_calmux(CALMUX_CAL_MUX2 | CALMUX_CAL_MUX4 | CALMUX_CAL_65MV
			 | CALMUX_CAL_45MV | CALMUX_VCAL_DMM | CALMUX_GND_DMM);
      PRINTERR();
      sleep(2);
      if (!ret)
	ret = dmm_read(dmm, &tmp);
      PRINTERR();
      if (ret) {
	log_error("Error testing if dmm is attached!\n");
	logerror(VXILOG_CRIT, "Error finding dmm");
	return 1;
      }
      else {
/* 	printf("testing GND\n"); */
	ret = !(tmp > -0.001 && tmp < 0.001);
	if (ret)
	  break;
      }
      PRINTERR();
    }
    
    count--;
    PRINTERR();
  }

  if (ret) {
    log_error("DMM does not appear to be attached!\n");
    logerror(VXILOG_CRIT, "DMM does not appear to be attached");
  }

  return ret && !ignore_boundry;
}

static int reset_system(EX1048_CLIENT *clprog, EX1048_CLIENT *clcore) {
  int ret = 0;
  EX1048_RPCresult result;

  if (!ret)
    ret = enable_fact_mode(1);

  if (!ret)
    ret = disable_mux_discharge(0);

  if (!ret)
    ret = set_freq(clprog, 0);

  if (!ret)
    ret = set_calmux(CALMUX_CAL_MUX1 | CALMUX_CAL_MUX2 | CALMUX_CAL_MUX3 |
		     CALMUX_CAL_MUX4 | CALMUX_CAL_65MV | CALMUX_CAL_45MV);

  if (!ret)
    ret = reset_trig(clprog);

  if (!ret)
    ret = report_cjc(clprog, 0);

  if (!ret)
    ret = set_conv_mV(clprog);

  if (!ret)
    ret = cjc_conv_mV(0);

  if (!ret)
    ret = set_cal_mode(0);

  if (!ret)
    ret = reset_cjc_channels(clprog);

  if (!ret)
    ret = enable_channels(clprog, 1);

  if (!ret)
    ret = enable_fact_mode(0);

  if (!ret)
    ret = reset_0x00000101(clprog->lid, &result, clprog->rpc_cl);
  if (!ret)
    ret = result.error_code;
  return ret;
}

