/*****************************************************************************
Copyright VXI Technoloy, Inc., 2004
 
Author:     $Author: benf $    
Date:       $Date: 2006/01/31 20:23:34 $
Revision:   $Revision: 1.2.38.1 $
 
$Header: /home/shared/cvsroot/ex1048/uClinux-dist/user/ex1048/client/findinstr.c,v 1.2.38.1 2006/01/31 20:23:34 benf Exp $
 
*****************************************************************************/

/* command line program to find all EX1048 Servers on the local network */

#include<stdio.h>
#include<unistd.h>
#include"ex1048.h"
#include"libex1048.h"
#include"scpilib.h"
#include<sys/time.h>
#include<sys/socket.h>
#include<netinet/in.h>
#include<arpa/inet.h>
#include<rpc/pmap_clnt.h>

bool_t reply_handler( char *out, struct sockaddr_in *addr );

#define MAX_ADDR 1024
#define TIMEOUT_SECS 5

#define VXIPROG 0x0607AF
#define VXIPROGVERS 1

#define SCPI_BUF_SIZE 256
#define DEVICE_MANUFACTURER "VXI Technology Inc."
#define DEVICE_MODEL "EX1048"

#define IO_TIMEOUT 5000 //this is needed because of a change in scpilib which means scpi_read requires a timeout. Value in milliseconds.

/* globals used by RPC call back */
struct timeval g_ti; 
struct timezone g_tiz;
int g_max_addr;
int *g_num_found;
struct in_addr *g_addresses;
int g_timeout_sec;

int findinstr_main( int argc, char **argv )
{
#ifdef __uClinux__
  fprintf( stderr, "This utility is not currently supported under "
	   "uClinux due to issues with the portmap daemon\n");
  fprintf( stderr, "Sorry :-(\n");
  return 1;
#else
  int i;
  struct in_addr addr[MAX_ADDR];
  int num_found;
  
  printf("Searching");
  fflush(stdout);
  if( findinstr_cmd( MAX_ADDR, &num_found, addr, TIMEOUT_SECS ) == 0 ) {
    printf("\n");
    if( num_found > 0 ) {
      printf("The following EX1048 Instruments were found:\n");
      for( i = 0; i < num_found; i++ ) {
	printf("%s\n", inet_ntoa( addr[i] ));
      }
      return 0;
    } else {
      printf("No Servers Found!\n");
    }
  } else {
    printf("\n");
    fprintf(stderr, "Error find findistr_cmd\n");
    return -1;
  }
  return 0;
#endif
}

int findinstr_cmd( int max_addr, int *num_found, 
		   struct in_addr *addresses, int timeout_secs)
{
  enum clnt_stat result;
 
  /* set up global pointer variables */
  g_max_addr = max_addr;
  g_num_found = num_found;
  g_addresses = addresses;
  
  /* initialize count to zero */
 *g_num_found = 0;
  
 /* get the current time */
  if( gettimeofday(&g_ti, &g_tiz) < 0 ){
    fprintf( stderr,"gettimeofday failed\n");
    exit(-1);
  }

 /* set our timeout */
  g_ti.tv_sec += timeout_secs;

  /* make the client broadcast call */
  result = clnt_broadcast( VXIPROG, VXIPROGVERS, NULLPROC, 
			   (xdrproc_t) xdr_void, (char *) NULL, 
			   (xdrproc_t) xdr_void, (char *) NULL, 
			   reply_handler);

  return 0;
}

bool_t device_is_an_ex1048 (const char *address) {
  struct scpi_handle *scpi;
  char *buf;
  char *manufacturer, *model, *serial_num;
  int major_version, minor_version, build_version;
  
  scpi = scpi_create();
  if (!scpi) {
    return 0;
  }
  
  if (SCPI_ERROR_SUCCESS != scpi_connect(scpi, address, "")) {
    scpi_destroy(scpi);
    return 0;
  }
  
  /* make the *IDN request */
  if (SCPI_ERROR_SUCCESS != scpi_write(scpi, "*IDN?\n")) {
    scpi_destroy(scpi);
  }
  
  /* allocate the receive buffer */
  buf = (char *)malloc(SCPI_BUF_SIZE + 1);
  if (!buf) {
    scpi_destroy(scpi);
    return 0;
  }
  
  /* get the result of the *IDN request */
  if (SCPI_ERROR_SUCCESS != scpi_read(scpi, buf, SCPI_BUF_SIZE, IO_TIMEOUT)) {
    scpi_destroy(scpi);
    free(buf);
    return 0;
  }
  
  /* allocate the manufacturer and model buffers */
  if (NULL == (manufacturer = (char *)malloc(SCPI_BUF_SIZE + 1))) {
    scpi_destroy(scpi);
    free(buf);
    return 0;
  }
  if (NULL == (model = (char *)malloc(SCPI_BUF_SIZE + 1))) {
    scpi_destroy(scpi);
    free(buf);
    free(manufacturer);
    return 0;
  }
  if (NULL == (serial_num = (char *)malloc(SCPI_BUF_SIZE + 1))) {
    scpi_destroy(scpi);
    free(buf);
    free(manufacturer);
    free(model);
    return 0;
  }
  
  sscanf(buf, "%255[^,],%255[^,],%255[^,],%d.%d.%d\n", manufacturer, model, serial_num,
	 &major_version, &minor_version, &build_version);
  
  if (strcmp(manufacturer, DEVICE_MANUFACTURER) || strcmp(model, DEVICE_MODEL)) {
    scpi_destroy(scpi);
    free(buf);
    free(serial_num);
    free(manufacturer);
    free(model);
    return 0;
  }
  
  printf("(v%d.%d.%d)", major_version, minor_version, build_version);
  fflush(stdout);
  
  scpi_destroy(scpi);
  free(buf);
  free(manufacturer);
  free(model);
  return 1;
}

bool_t reply_handler( char *out, struct sockaddr_in *addr )
{
  struct timeval ti; 
  struct timezone tiz;
  int found;
  int i;

  /* get the current time */
  if( gettimeofday(&ti, &tiz) < 0 ){
    fprintf( stderr,"gettimeofday failed\n");
    exit(-1);
  }
  printf(".");
  fflush(stdout);

  /* look for the address in our array to see if
     we've heard from that device already */
  found = 0;
  for( i = 0; i < *g_num_found; i++ ) {
    if( g_addresses[ i ].s_addr == addr->sin_addr.s_addr ) {
      found = 1;
      break;
    }
  }
  if( !found ) {
    /* is this really an EX1048 device? */
    if (device_is_an_ex1048(inet_ntoa(addr->sin_addr))) {
      /* add the address to our array */
      g_addresses[ *g_num_found ].s_addr = addr->sin_addr.s_addr;
      *g_num_found += 1;
    }
  }
  
  /* if we're full, or our timeout has expired, we're done */
  if( *g_num_found >= g_max_addr ) {
    printf("list full\n");
    return 1;
  } else if( ti.tv_sec >= g_ti.tv_sec ) {
    return 1;
  } else {
    return 0;
  }
}
	  
/*****************************************************************************
Revision History
 
$Log: findinstr.c,v $
Revision 1.2.38.1  2006/01/31 20:23:34  benf
Added support for multiple DMM selection as on the 1629

Revision 1.2  2004/09/22 17:52:06  gbernhardt
merging alpha back onto the root (bug #79)

Revision 1.1.2.8  2004/08/15 20:18:01  nbarendt
updated findinstr to use build version properly

Revision 1.1.2.7  2004/08/03 17:42:34  gbernhardt
re-added some local variables that were incorrectly removed

Revision 1.1.2.6  2004/07/15 19:39:25  gbernhardt
removed unused variables

Revision 1.1.2.5  2004/07/14 15:22:24  gbernhardt
modified findinstr to use scpilib to recognize whether the devices it finds are actually EX1048s

Revision 1.1.2.4  2004/07/09 19:32:05  nbarendt
removed debugging printfs

Revision 1.1.2.3  2004/04/22 17:15:15  nbarendt
removed debugging printfs

Revision 1.1.2.2  2004/04/21 23:10:49  nbarendt
findinstr now works on non-uClinux systems

Revision 1.1.2.1  2004/04/20 13:56:30  nbarendt
moved from lib dir

Revision 1.1  2004/03/12 16:47:40  nbarendt
initial checkin



*****************************************************************************/
