/*****************************************************************************
Copyright VXI Technoloy, Inc., 2004
 
Author:     $Author: jmiller $    
Date:       $Date: 2005/02/16 19:54:02 $
Revision:   $Revision: 1.6 $
 
$Header: /home/shared/cvsroot/ex1048/uClinux-dist/user/ex1048/client/libex1048.h,v 1.6 2005/02/16 19:54:02 jmiller Exp $
 
*****************************************************************************/

#ifndef LIBEX1048_HEADER_
#define LIBEX1048_HEADER_

#include"ex1048.h"

#define EX1048_CLIENT_SUCCESS_STR "Success!\n"
#define EX1048_CLIENT_FAILURE_STR "Failure!\n"

/* EX1048 client structure */
typedef struct {
  CLIENT *rpc_cl;
  int lid;
  int manual_lid; /* was the lid passed in manually? */
} EX1048_CLIENT;

/* some utility functions */
EX1048_CLIENT *ex1048_clnt_create(const char *host, const char *netid, int version, int lid );
void ex1048_clnt_destroy(EX1048_CLIENT *cl);

#define SERIAL_NUMBER_LENGTH 32
#define IPADDRS_LENGTH 64 /* "255.255.255.255" is the longest IP address (15
			   * characters), so this is way more space than we
			   * should need for our two IP addresses. */
#define MAC_LENGTH 32
#define DEFAULT_MAC_ADDRESS "00:0D:3F:00:FF:FF"
/* First 3 octets of VXI assigned MAC address block */
#define MAC_ADDRESS_BLOCK   "00:0D:3F"
/* Require the device ID serial number to be 6 decimal digits */
#define REQUIRED_SERIAL_NUMBER_LENGTH   6
/* The web page displays the following text when instructing a user on the */
/*   proper serial number and MAC address formats.                         */
#define REQUIRED_SERIAL_NUMBER_FORMAT   "dddddd"
#define REQUIRED_MAC_ADDRESS_FORMAT     "00:0D:3F:hh:hh:hh"

/* flash environment variable functions */
const char *devinfo_get_serial (void);
const char *devinfo_get_ipaddrs (void);
const char *devinfo_get_mac (void);

/* function to build a floating-point formatting string automatically */
const char *build_float_format (EX1048_RPCEUConversion conv);

#define RANGE_DELIM '-'

void show_usage(void);

int ex1048client_main( int argc, char **argv );
int ex1048client_cmd( void );

int getscanlist_main( int argc, char **argv );
int getscanlist_cmd( EX1048_CLIENT *cl,  int *id, int *num_channels, 
		     int *channels);

int setscanlist_main( int argc, char **argv );
int setscanlist_cmd( EX1048_CLIENT *cl, int id, int num_channels, int channels[],
		     EX1048_RPCresult  *result);

int getstream_main(int argc, char **argv);

#define CONV_NAME_LEN 50
extern const char conv_names[16][CONV_NAME_LEN];
int setconversions_main( int argc, char **argv );
int setconversions_cmd( EX1048_CLIENT *cl, int num_channels, int channels[], 
			EX1048_RPCEUConversion conversions[],
			EX1048_RPCresult  *rpc_result);

int getconversions_main( int argc, char **argv );
int getconversions_cmd( EX1048_CLIENT *cl, int *num_channels, int *channels, 
			EX1048_RPCEUConversion *conversions);

int readfifo_main( int argc, char **argv );
int readfifo_cmd( EX1048_CLIENT *cl, int num_blocks, int *fifo_count,
		  EX1048_RPCFIFOArray *rpc_result);

int findinstr_main( int argc, char **argv );
int findinstr_cmd( int max_addr, int *num_found, 
		   struct in_addr *addresses, int timeout_secs);

int triginit_main( int argc, char **argv );
int triginit_cmd( EX1048_CLIENT *cl, EX1048_RPCresult  *rpc_result );

int exlock_main(int argc, char **argv);
int exlock_cmd(EX1048_CLIENT *cl, EX1048_RPCresult *rpc_result);

int exunlock_main(int argc, char **argv);
int exunlock_cmd(EX1048_CLIENT *cl, EX1048_RPCresult *rpc_result);

int exbreaklock_main(int argc, char **argv);
int exbreaklock_cmd(EX1048_CLIENT *cl, EX1048_RPCresult *rpc_result);

int selfcal_main(int argc, char **argv);
int selfcal_cmd(EX1048_CLIENT *cl, int magic, EX1048_RPCSelfCalResult *rpc_result);

int selfcalclear_main(int argc, char **argv);
int selfcalclear_cmd(EX1048_CLIENT *cl, EX1048_RPCresult *rpc_result);

int selfcalstore_main(int argc, char **argv);
int selfcalstore_cmd(EX1048_CLIENT *cl, EX1048_RPCresult *rpc_result);

int selfcalload_main(int argc, char **argv);
int selfcalload_cmd(EX1048_CLIENT *cl, EX1048_RPCresult *rpc_result);

int abort_main( int argc, char **argv );
int abort_cmd( EX1048_CLIENT *cl, EX1048_RPCresult  *rpc_result );

int resetfifo_main( int argc, char **argv );
int resetfifo_cmd( EX1048_CLIENT *cl, EX1048_RPCresult  *rpc_result );

int configfifo_main( int argc, char **argv );
int configfifo_cmd( EX1048_CLIENT *cl, int *cjc_mode, int *timestamp_mode, int *count_mode, int *temp_units, int *blocking_mode,
		    EX1048_RPCFIFOFormat  *rpc_result);

int exreset_main( int argc, char **argv );
int exreset_cmd( EX1048_CLIENT *cl, EX1048_RPCresult  *rpc_result );

int trigsetcount_main( int argc, char **argv );
int trigsetcount_cmd( EX1048_CLIENT *cl, uint32_t count, EX1048_RPCresult *rpc_result );

int armsetcount_main( int argc, char **argv );
int armsetcount_cmd( EX1048_CLIENT *cl, uint32_t count, EX1048_RPCresult *rpc_result );

int trigsetdelay_main( int argc, char **argv );
int trigsetdelay_cmd( EX1048_CLIENT *cl, double delay, EX1048_RPCresult *rpc_result );

int armsetdelay_main( int argc, char **argv );
int armsetdelay_cmd( EX1048_CLIENT *cl, double delay, EX1048_RPCresult *rpc_result );

int triggetcount_main( int argc, char **argv );
int triggetcount_cmd( EX1048_CLIENT *cl, EX1048_RPCTrigCount *rpc_result );

int armgetcount_main( int argc, char **argv );
int armgetcount_cmd( EX1048_CLIENT *cl, EX1048_RPCTrigCount *rpc_result );

int triggetdelay_main( int argc, char **argv );
int triggetdelay_cmd( EX1048_CLIENT *cl, EX1048_RPCTrigDelay *rpc_result );

int armgetdelay_main( int argc, char **argv );
int armgetdelay_cmd( EX1048_CLIENT *cl, EX1048_RPCTrigDelay *rpc_result );

int trigsettimer_main( int argc, char **argv );
int trigsettimer_cmd( EX1048_CLIENT *cl, double timer_period, EX1048_RPCresult *rpc_result );

int triggettimer_main( int argc, char **argv );
int triggettimer_cmd( EX1048_CLIENT *cl, EX1048_RPCTrigTimer *rpc_result );

extern const char trig_src_names[][10];

int initsetcont_main( int argc, char **argv );
int initsetcont_cmd( EX1048_CLIENT *cl, int cont, EX1048_RPCresult *rpc_result);

int initgetcont_main( int argc, char **argv );
int initgetcont_cmd( EX1048_CLIENT *cl, EX1048_RPCcontinuous *rpc_result);

int trigsetinf_main( int argc, char **argv );
int trigsetinf_cmd( EX1048_CLIENT *cl, int inf, EX1048_RPCresult *rpc_result);

int triggetinf_main( int argc, char **argv );
int triggetinf_cmd( EX1048_CLIENT *cl, EX1048_RPCTrigInf *rpc_result);

int armsetinf_main( int argc, char **argv );
int armsetinf_cmd( EX1048_CLIENT *cl, int inf, EX1048_RPCresult *rpc_result);

int armgetinf_main( int argc, char **argv );
int armgetinf_cmd( EX1048_CLIENT *cl, EX1048_RPCTrigInf *rpc_result);

int trigsetsrc_main( int argc, char **argv );
int trigsetsrc_cmd( EX1048_CLIENT *cl, EX1048_RPCTrigSrc *trig_src, EX1048_RPCresult *rpc_result );

int armsetsrc_main( int argc, char **argv );
int armsetsrc_cmd( EX1048_CLIENT *cl, EX1048_RPCTrigSrc *trig_src, EX1048_RPCresult *rpc_result );

int triggetsrc_main( int argc, char **argv );
int triggetsrc_cmd( EX1048_CLIENT *cl, EX1048_RPCTrigSrc *rpc_result );

int armgetsrc_main( int argc, char **argv );
int armgetsrc_cmd( EX1048_CLIENT *cl, EX1048_RPCTrigSrc *rpc_result );

int softwarearm_main( int argc, char **argv );
int softwarearm_cmd( EX1048_CLIENT *cl, EX1048_RPCresult *rpc_result );

int softwaretrigger_main( int argc, char **argv );
int softwaretrigger_cmd( EX1048_CLIENT *cl, EX1048_RPCresult *rpc_result );

int setfreqs_main( int argc, char **argv );
int setfreqs_cmd( EX1048_CLIENT *cl, int num_channels, int channels[], 
		  double freqs[], EX1048_RPCresult  *rpc_result);

int getfreqs_main( int argc, char **argv );
int getfreqs_cmd( EX1048_CLIENT *cl, int *num_channels, 
		  int *channels, double *freqs);

int diosetoutput_main(int argc, char **argv);
int diosetoutput_cmd(EX1048_CLIENT *cl, unsigned char value, EX1048_RPCresult *rpc_result);

int diogetoutput_main(int argc, char **argv);
int diogetoutput_cmd(EX1048_CLIENT *cl, EX1048_RPCByte *rpc_result);

int diogetinput_main(int argc, char **argv);
int diogetinput_cmd(EX1048_CLIENT *cl, EX1048_RPCByte *rpc_result);

int diogetenable_main(int argc, char **argv);
int diogetenable_cmd(EX1048_CLIENT *cl, EX1048_RPCByte *rpc_result);

int diosendpulse_main(int argc, char **argv);
int diosendpulse_cmd(EX1048_CLIENT *cl, unsigned char value, EX1048_RPCresult *rpc_result);

int diosetenable_main(int argc, char **argv);
int diosetenable_cmd(EX1048_CLIENT *cl, unsigned char value, EX1048_RPCresult *rpc_result);

int vtbsetoutput_main(int argc, char **argv);
int vtbsetoutput_cmd(EX1048_CLIENT *cl, unsigned char value, EX1048_RPCresult *rpc_result);

int vtbgetoutput_main(int argc, char **argv);
int vtbgetoutput_cmd(EX1048_CLIENT *cl, EX1048_RPCByte *rpc_result);

int vtbgetinput_main(int argc, char **argv);
int vtbgetinput_cmd(EX1048_CLIENT *cl, EX1048_RPCByte *rpc_result);

int vtbgetenable_main(int argc, char **argv);
int vtbgetenable_cmd(EX1048_CLIENT *cl, EX1048_RPCByte *rpc_result);

int vtbsendpulse_main(int argc, char **argv);
int vtbsendpulse_cmd(EX1048_CLIENT *cl, unsigned char value, EX1048_RPCresult *rpc_result);

int vtbsetenable_main(int argc, char **argv);
int vtbsetenable_cmd(EX1048_CLIENT *cl, unsigned char value, EX1048_RPCresult *rpc_result);

int setlimits_main(int argc, char **argv);
int setlimits_cmd(EX1048_CLIENT *cl, EX1048_RPCLimits limits, EX1048_RPCresult *result);

int getlimits_main(int argc, char **argv);
int getlimits_cmd(EX1048_CLIENT *cl, int set, EX1048_RPCLimits *rpc_result);

int setlimitenables_main(int argc, char **argv);
int setlimitenables_cmd(EX1048_CLIENT *cl, EX1048_RPCLimitEnables limits, EX1048_RPCresult *result);

int getlimitenables_main(int argc, char **argv);
int getlimitenables_cmd(EX1048_CLIENT *cl, EX1048_RPCLimitEnables *rpc_result);

int getlimitstatus_main(int argc, char **argv);
int getlimitstatus_cmd(EX1048_CLIENT *cl, EX1048_RPCLimitStatus *rpc_result);

int setuserconv_main(int argc, char **argv);
int setuserconv_cmd(EX1048_CLIENT *cl, EX1048_RPCUserConversion *conv, EX1048_RPCresult *result);

int getuserconv_main(int argc, char **argv);
int getuserconv_cmd(EX1048_CLIENT *cl, int set, EX1048_RPCUserConversion *rpc_result);

int getcalstatus_main(int argc, char **argv);
int getcalstatus_cmd(EX1048_CLIENT *cl, EX1048_RPCCalibrationStatus *rpc_result);

int geterrorstatus_main(int argc, char **argv);
int geterrorstatus_cmd(EX1048_CLIENT *cl, EX1048_RPCErrorStatus *rpc_result);

int getcalrunning_main(int argc, char **argv);
int getcalrunning_cmd(EX1048_CLIENT *cl, EX1048_RPCCalibrationRunning *rpc_result);

int getfirmwareversion_main(int argc, char **argv);
int getfirmwareversion_cmd(EX1048_CLIENT *cl, EX1048_RPCFirmwareVersion *rpc_result);

int getrpcversion_main(int argc, char **argv);
int getrpcversion_cmd(EX1048_CLIENT *cl, EX1048_RPCVersions *rpc_result);

int setcjctemperatures_main(int argc, char **argv);
int setcjctemperatures_cmd(EX1048_CLIENT *cl, EX1048_RPCCJCTemperatures *temps, EX1048_RPCresult *rpc_result);

int getcjctemperatures_main(int argc, char **argv);
int getcjctemperatures_cmd(EX1048_CLIENT *cl, EX1048_RPCCJCTemperatures *rpc_result);

int setcjcenables_main(int argc, char **argv);
int setcjcenables_cmd(EX1048_CLIENT *cl, EX1048_RPCCJCEnables *enables, EX1048_RPCresult *rpc_result);

int getcjcenables_main(int argc, char **argv);
int getcjcenables_cmd(EX1048_CLIENT *cl, EX1048_RPCCJCEnables *rpc_result);

int selftest_main(int argc, char **argv);
int selftest_cmd(EX1048_CLIENT *cl, EX1048_RPCSelfTestResult *rpc_result);

int getsystemstatus_main(int argc, char **argv);
int getsystemstatus_cmd(EX1048_CLIENT *cl, EX1048_RPCSystemStatus *rpc_result);

int setledstatus_main(int argc, char **argv);
int setledstatus_cmd(EX1048_CLIENT *cl, EX1048_RPCUserLEDStatus *status, EX1048_RPCresult *rpc_result);

int getledstatus_main(int argc, char **argv);
int getledstatus_cmd(EX1048_CLIENT *cl, EX1048_RPCUserLEDStatus *rpc_result);

int setauxledstatus_main(int argc, char **argv);
int setauxledstatus_cmd(EX1048_CLIENT *cl, EX1048_RPCUserAuxLEDStatus *status, EX1048_RPCresult *rpc_result);

int getauxledstatus_main(int argc, char **argv);
int getauxledstatus_cmd(EX1048_CLIENT *cl, EX1048_RPCUserAuxLEDStatus *rpc_result);

int getchannelunits_main(int argc, char **argv);
int getchannelunits_cmd(EX1048_CLIENT *cl, EX1048_RPCChannelUnits *rpc_result);

int getserial_main(int argc, char **argv);
int getserial_cmd(EX1048_CLIENT *cl, EX1048_RPCSerialNumber *rpc_result);

int caldate_main(int argc, char **argv);
int caldate_cmd(EX1048_CLIENT *cl, EX1048_RPCCalibrationDate *rpc_result);

int lidlock_main(int argc, char **argv);

#endif


/*****************************************************************************
Revision History
 
$Log: libex1048.h,v $
Revision 1.6  2005/02/16 19:54:02  jmiller
Changes to resolve bug 476 - modify MAC and SN CGI for ease of production.

Revision 1.5  2005/01/06 19:42:03  gbernhardt
added an 'IP Address' field to the index page that shows all of the device's IP addresses. (bug #438)

Revision 1.4  2004/09/30 19:27:18  pmecklenburg
added lidlock

Revision 1.3  2004/09/30 19:16:25  gbernhardt
fixed formatting in readfifo client program to match that in the web interface (bug #283)

Revision 1.2  2004/09/22 17:52:06  gbernhardt
merging alpha back onto the root (bug #79)

Revision 1.1.2.34  2004/09/21 14:34:26  gbernhardt
added link IDs (bug #227)

Revision 1.1.2.33  2004/09/07 21:53:31  pmecklenburg
updated selfcal_cmd to take magic and return selfcal data

Revision 1.1.2.32  2004/08/30 15:18:05  gbernhardt
added enable flags to channel 0 of the limits (they don't do anything yet) (bug #147)

Revision 1.1.2.31  2004/08/23 21:37:38  pmecklenburg
added getserial and caldate

Revision 1.1.2.30  2004/08/21 20:07:52  gbernhardt
overhauled RPC system to use new program number (bug #68)

Revision 1.1.2.29  2004/08/18 14:37:30  gbernhardt
added an RPC and userland app to get the channel units (bug #145)

Revision 1.1.2.28  2004/08/10 19:56:42  gbernhardt
added support for aux LED (bug #86)

Revision 1.1.2.27  2004/08/10 18:33:06  gbernhardt
added driver interface for controlling LEDs (bug #86)

Revision 1.1.2.26  2004/08/09 15:27:31  gbernhardt
added 'get system status' and 'get limit status' userland utilities (bugs #20 and #62)

Revision 1.1.2.25  2004/08/06 14:43:11  pmecklenburg
added geterrorstatus

Revision 1.1.2.24  2004/08/03 21:40:55  gbernhardt
added software arm and trigger (bug #63)

Revision 1.1.2.23  2004/08/03 20:10:06  pmecklenburg
added selfcal clients, locking clients and getcalrunning client

Revision 1.1.2.22  2004/08/03 14:58:58  gbernhardt
added DIO pulse and VTB pulse command line programs

Revision 1.1.2.21  2004/08/01 17:14:59  gbernhardt
added initial (incomplete) self-test

Revision 1.1.2.20  2004/07/30 18:48:47  gbernhardt
fixed a misnamed parameter

Revision 1.1.2.19  2004/07/30 17:49:05  gbernhardt
added preliminary VTB support (bug #56)

Revision 1.1.2.18  2004/07/29 18:04:31  gbernhardt
added blocking mode control to config_fifo (bug #50)

Revision 1.1.2.17  2004/07/29 17:03:27  gbernhardt
added temperature units setting (bug #18)

Revision 1.1.2.16  2004/07/28 22:00:08  gbernhardt
added user-defined CJC temperature utilities

Revision 1.1.2.15  2004/07/26 18:23:19  gbernhardt
added firmware version and RPC version RPC calls (bug #46)

Revision 1.1.2.14  2004/07/26 14:49:51  gbernhardt
added support for multiple limit sets

Revision 1.1.2.13  2004/07/19 21:08:17  gbernhardt
added calibration status userland utility

Revision 1.1.2.12  2004/07/16 20:35:03  gbernhardt
modified setuserconv RPC and userland to only take one argument

Revision 1.1.2.11  2004/07/09 19:33:28  nbarendt
added string length #defines as well as device info function declaration

Revision 1.1.2.10  2004/07/07 15:36:56  gbernhardt
added user-defined conversions

Revision 1.1.2.9  2004/06/28 18:52:57  gbernhardt
merged alpha alpha-trigger

Revision 1.1.2.8.2.3  2004/06/24 14:46:56  pmecklenburg
changed delay from int (clock ticks) to double (sec)

Revision 1.1.2.8.2.2  2004/06/23 13:28:28  pmecklenburg
add clients for new tirgger model (arm stuff, inf, delay, etc)

Revision 1.1.2.8.2.1  2004/06/22 18:52:05  pmecklenburg
RPC name change for updated trigger system

Revision 1.1.2.8  2004/06/18 17:56:14  pmecklenburg
changed ex1048_clnt_create to take const char *

Revision 1.1.2.7  2004/06/17 14:10:08  gbernhardt
added getlimits and setlimits

Revision 1.1.2.6  2004/06/03 20:07:08  gbernhardt
added digital IO RPC and userland tools

Revision 1.1.2.5  2004/05/11 12:57:34  nbarendt
removed CGI function signatures - they are in the CGI header now

Revision 1.1.2.4  2004/04/21 14:24:57  nbarendt
added range delimiter symbol string

Revision 1.1.2.3  2004/04/20 13:56:31  nbarendt
moved from lib dir

Revision 1.1.2.1  2004/04/20 13:49:39  nbarendt
moved from client subdir

Revision 1.1.2.1  2004/04/20 13:16:05  nbarendt
moved from parent directory

Revision 1.3.2.2  2004/04/20 12:42:34  nbarendt
added count support

Revision 1.3.2.1  2004/04/13 19:55:35  nbarendt
added several new interfaces

Revision 1.3  2004/04/12 14:11:46  nbarendt
fixed header protection
added exreset command

Revision 1.2  2004/03/12 16:45:54  nbarendt
added new client functions

Revision 1.1  2004/02/12 19:18:04  nbarendt
initial versions


*****************************************************************************/
