/*****************************************************************************
Copyright VXI Technoloy, Inc., 2004
 
Author:     $Author: gbernhardt $    
Date:       $Date: 2004/10/01 21:38:55 $
Revision:   $Revision: 1.6 $
 
$Header: /home/shared/cvsroot/ex1048/uClinux-dist/user/ex1048/client/readfifo.c,v 1.6 2004/10/01 21:38:55 gbernhardt Exp $
 
*****************************************************************************/

/* command line program to get data from the FIFO */

#include<stdio.h>
#include<rpc/rpc.h>
#include<unistd.h>
#include<getopt.h>
#include<stdlib.h>
#include<sys/time.h>
#include"libex1048.h"
#include"ex1048.h"
#include"ex1048_errors.h"
 
int fdump_data_page_as_csv( FILE *fout, EX1048_RPCSampleData *dp,  int channels[], EX1048_RPCEUConversion *conversions);
int fdump_csv_header( FILE *fout, EX1048_RPCSampleData *dp,  int channels[]);


#define MICRO_SECS_PER_SEC 1000000.0F
//#define FP_VAL_STR "%+8.8E"
#define FP_VAL_STR "%+8.2f"

struct precise_tm {
  struct tm *base;
  u_int usecs; /* microseconds */
};

static int print_precise_time (FILE *fout, u_int secs, u_int usecs) {
  char day_of_week[4], month[4];
  struct precise_tm t;
  time_t base_time;

  base_time = secs;
  t.base = localtime(&base_time);
  t.usecs = usecs;

  sscanf(asctime(t.base), "%s %s", day_of_week, month);
  fprintf(fout, "%s %s %02i %02i %02i:%02i:%02f\n",
	  day_of_week,
	  month,
	  t.base->tm_mday,
	  1900 + t.base->tm_year,
	  t.base->tm_hour,
	  t.base->tm_min,
	  (double)t.base->tm_sec + (double)t.usecs / MICRO_SECS_PER_SEC);

  return 0;
}

int readfifo_main( int argc, char **argv )
{
  const char optstring[]="tuh:vc:p:dxsq";
  const char defaulthost[] = "localhost";
  const char defaultnetid[] = "udp";
  char *host = (char *) defaulthost;
  char *netid =(char *) defaultnetid;
  int verbose = 0; /* default non-verbose */
  int csvformat = 0;
  int hexdisplaycounts = 0; /* default to signed decimal display for counts */
  EX1048_CLIENT *cl;
  int c;
  EX1048_RPCFIFOArray rpc_data;
  EX1048_RPCEUConversion conversions[EX1048_MAX_NUM_CHANNELS];
  unsigned long i;
  int pages = 0; /* default to get zero pages of data per request (return FIFO count)*/
  int count = 1; /* default to one request */
  int continuous = 0;
  int discard = 0; /* display the data, by default */
  int display_throughput = 0; /* display KB/sec */
  int j, k;
  unsigned long starts = 0, startus = 0, stops = 0, stopus = 0;
  struct timeval ti; 
  struct timezone tiz;
  double start_t = 0.0, stop_t = 0.0, duration = 0.0;
  double bytes_xfered = 0;
  FILE *fout = stdout;
  char *filename;
  int fcount;
  uint16_t *timestamps;
  int id = EX1048_CURRENT_SCANLIST; /* default to setting the current scanlist */
  int num_channels; 
  int channels[EX1048_MAX_SCANLIST_LENGTH];
  int first_loop = 1;
  
  memset(&rpc_data, 0, sizeof(rpc_data));
  memset(conversions, 0, EX1048_MAX_NUM_CHANNELS * sizeof(EX1048_RPCEUConversion));
  
  /* defaults to localhost via UDP */
  while ((c=getopt(argc,argv,optstring))!=-1) {
    switch(c)	{
    case 't': netid = "tcp"; break;
    case 'u': netid = "udp"; break;
    case 'h': host = optarg; break;
    case 'v': verbose = 1; break;
    case 'c': count = atoi(optarg); if( count == -1) continuous = 1; break;
    case 'p': 
      pages = atoi(optarg); 
      if( pages == -1 ) pages = EX1048_MAX_FIFO_RESPONSE; 
      break;
    case 'd': discard = 1; break;
    case 'x': display_throughput = 1; break;
    case 's': csvformat = 1; break;
    case 'q': hexdisplaycounts = 1; break;
    default: show_usage();
    }
  }
  
  /* see what arguments are left */
  if( ( argc - optind ) == 1 ) {
    /* we should output to a file instead of stdout */
    filename = argv[optind];
    fout = fopen( filename, "w");
    if( fout == NULL ) {
      fprintf(stderr, "Error openig %s for writing\n", filename );
      show_usage();
    }
  }

  /* create the client handle */
  cl = ex1048_clnt_create( host, netid, EX1048PROGVERS, LID_NONE );
  if (!cl) {
    fprintf(fout, "Failed to connect to server\n");
    exit(1);
  }
  
  /* get the current scanlist configuration */
  if( pages ) {
    if( getscanlist_cmd( cl,  &id, &num_channels, channels ) == EX1048_SUCCESS ) {
      if( verbose ) fprintf( fout, "Retrieved Current Scanlst\n");
    } else {
      fprintf(stderr, "error getting current scanlist\n");
      exit(1);
    }
  }
  
  /* If we're retrieving pages, get the conversions. They're used later to 
     build formatting strings for the data. */
  if (pages) { 
    if (getconversions_cmd(cl, &num_channels, channels,conversions)) {
      fprintf(stderr, "error getting channel conversions\n");
      exit(1);
    }
  }
  
  i = 0;
  while( ( i < count ) || (continuous) )
    {
      if (gettimeofday(&ti, &tiz) < 0)
	{
	  exit(10);
	}
      starts  = ti.tv_sec;
      startus = ti.tv_usec;
      if( readfifo_cmd( cl, pages, &fcount, &rpc_data ) == EX1048_SUCCESS )
	{
	  if (gettimeofday(&ti, &tiz) < 0)
	    {
	      perror("tcp/udpblast time:");
	      exit(1);
	    }
	  stops  = ti.tv_sec;
	  stopus = ti.tv_usec;
	  start_t = ( double ) starts + startus * 1.0E-6;
	  stop_t = ( double ) stops + stopus * 1.0E-6;
	  duration = stop_t - start_t;
	  
	  /* only count the real data payload */
	  bytes_xfered = 0;

	  if( verbose ) fprintf( fout, EX1048_CLIENT_SUCCESS_STR );
	  if( !pages) fprintf( fout,"FIFO Contains %8d pages\n", rpc_data.fifo_count);
	  if( ( !discard ) && ( rpc_data.fifo.fifo_len ) ){
	    if( !csvformat ) {
	      fprintf( fout, "PAGES:  %4d rcvd %4d rqstd %8d rmng\n",
		       rpc_data.fifo.fifo_len, pages, rpc_data.fifo_count);
	    } else if( first_loop ) { /* only output the header for the first data set */
	      first_loop = 0;
	      fdump_csv_header( fout, rpc_data.fifo.fifo_val[0], channels );
	    }
	  }
	  for( j = 0; j < rpc_data.fifo.fifo_len; j++ )
	    {
	      EX1048_RPCSampleData_p sampledata = rpc_data.fifo.fifo_val[j];

	      if( !discard ) {
		if( !csvformat ) {
		  fprintf(fout, "PAGE %4d (", j);
		  print_precise_time(fout, sampledata->ss_secs, sampledata->ss_usecs);
		  fprintf(fout, ")\n");
		  /*
		  fprintf( fout, "PAGE %4d (%10lu.%10lu)\n",
			   j,
			   sampledata->ss_secs,
			   sampledata->ss_usecs);
		  */
		  fprintf( fout, "----------\n");
		  if( sampledata->cjc.cjc_len ) {
		    fprintf( fout, "CJC\n");
		    fprintf( fout, "----------\n");
		    timestamps = ( uint16_t * ) sampledata
		      ->cjc_ticks.cjc_ticks_val;
		    for( k = 0; k < sampledata->cjc.cjc_len; k++ ) {
		      bytes_xfered += sizeof( float );
		      if( sampledata->cjc_ticks.cjc_ticks_len ) {
			fprintf( fout, "%2d " FP_VAL_STR  "%10u\n", k + EX1048_FIRST_CJC_CHANNEL, 
				 sampledata->cjc.cjc_val[k],
				 ntohs(timestamps[k]));
		      } else {
			fprintf( fout, "%2d " FP_VAL_STR "\n", k + EX1048_FIRST_CJC_CHANNEL, 
				 sampledata->cjc.cjc_val[k]);
		      }
		    }
		    fprintf( fout, "----------\n");
		  }
		  if( sampledata->cjc_counts.cjc_counts_len ) {
		    fprintf( fout, " CJC Counts    \n");
		    fprintf( fout, "----------\n");
		    for( k = 0; k < sampledata->cjc_counts.cjc_counts_len; k++ ) {
		      bytes_xfered += sizeof( float );
		      if( !hexdisplaycounts ) {
			fprintf( fout, "%2d %+8d\n", 
				 k + EX1048_FIRST_CJC_CHANNEL,  /* NAB??? need to query the CJC channel numbers */
				 sampledata->cjc_counts.cjc_counts_val[k]);
		      } else {
			fprintf( fout, "%2d %#8.8X\n", 
				 k + EX1048_FIRST_CJC_CHANNEL,  /* NAB??? need to query the CJC channel numbers */
				 (uint32_t) sampledata->cjc_counts.cjc_counts_val[k]);
		      }
		    
		    }
		    fprintf( fout, "----------\n");
		  }
		  timestamps = ( uint16_t * ) sampledata
		    ->x_ticks.x_ticks_val;
		  for( k = 0; k < sampledata->x.x_len; k++ ) {
		    bytes_xfered += sizeof( float );
		    fprintf(fout, "%2d ", channels[k]);
		    fprintf(fout, build_float_format(conversions[channels[k]]), sampledata->x.x_val[k]);
		    if( sampledata->x_ticks.x_ticks_len ) {
		      printf(" %10u", ntohs(timestamps[k]));
		    } else {
		      printf(" ");
		    }
		    if (sampledata->limits.limits_len) {
		      int index = channels[k] / 32;
		      int offset = channels[k] % 32;
		      if (sampledata->limits.limits_val[index + LIMITS_0_MIN_OFFSET] & (1 << offset))
			fprintf(fout, " [exceeded limit 0 min]");
		      if (sampledata->limits.limits_val[index + LIMITS_0_MAX_OFFSET] & (1 << offset))
			fprintf(fout, " [exceeded limit 0 max]");
		      if (sampledata->limits.limits_val[index + LIMITS_1_MIN_OFFSET] & (1 << offset))
			fprintf(fout, " [exceeded limit 1 min]");
		      if (sampledata->limits.limits_val[index + LIMITS_1_MAX_OFFSET] & (1 << offset))
			fprintf(fout, " [exceeded limit 1 max]");
		    }
		    fprintf(fout, "\n");
		  }
		  if( sampledata->x_counts.x_counts_len ) {
		    fprintf( fout, "----------\n");
		    fprintf( fout, "Counts    \n");
		    fprintf( fout, "----------\n");
		    for( k = 0; k < sampledata->x_counts.x_counts_len; k++ ) {
		      bytes_xfered += sizeof( float );
		      if( !hexdisplaycounts ) {
			fprintf( fout, "%2d %+8d\n", channels[k], 
				 sampledata->x_counts.x_counts_val[k]);
		      } else {
			fprintf( fout, "%2d %#8.8X\n", channels[k], 
				 (uint32_t) sampledata->x_counts.x_counts_val[k]);
		      }
		    
		    }
		    fprintf( fout, "----------\n");
		  }
		} else {
		  fdump_data_page_as_csv( fout, sampledata, channels, conversions );
		}
	      } else {
		bytes_xfered += sizeof( float ) 
		  * sampledata->x.x_len;
		bytes_xfered += sampledata->x_ticks.x_ticks_len ;
		bytes_xfered += sizeof( float ) 
		  * sampledata->cjc.cjc_len;
		bytes_xfered += sampledata->cjc_ticks.cjc_ticks_len ;
	      }
	    }
	  /* show transfer rate */
	  if( display_throughput )
	    {
	      if( !continuous )
		{
		  if( verbose ) 
		    fprintf(stderr,"%6.0f bytes of sample data transferred in %4.6f seconds\n",
			    bytes_xfered, duration);
		  fprintf(stderr,"%6.0f (KB/sec) %6.0f (pgs/sec) %d pgs (%.0f bytes) rcvd in %4.3f sec %d pages rmng\n", 
			  bytes_xfered / duration / 1024, 
			  rpc_data.fifo.fifo_len / duration, 
			  rpc_data.fifo.fifo_len, 
			  bytes_xfered,
			  duration,
			  rpc_data.fifo_count );
		}
	      /* only display throughput in continuous mode every few seconds */
	      else if( ( continuous ) && ( ( stops % 2 ) == 0 ) )
		{
		  if( verbose ) 
		    {
		      fprintf(stderr,"%6.0f bytes of sample data transferred in %4.6f seconds\n",
			      bytes_xfered, duration);
		      fprintf(stderr,"%6.0f (KB/sec)\n", bytes_xfered / duration / 1024);
		    }
		  else
		    fprintf(stderr,"%6.0f (KB/sec)\r", bytes_xfered / duration / 1024);
		}
	    }
	  /* free the rpc result data */
	  xdr_free( (xdrproc_t) xdr_EX1048_RPCFIFOArray, (char *) &rpc_data);
	}
      else
	{
	  fprintf( fout, EX1048_CLIENT_FAILURE_STR);
	  fprintf(stderr, "Error reading data %d\n", rpc_data.error_code);
	  exit(rpc_data.error_code);
	}
      i++;
    }
  
  /* destroy the client handle */
  ex1048_clnt_destroy( cl );
  return 0;
}


/* generate CSV column header line */
int fdump_csv_header( FILE *fout, EX1048_RPCSampleData *dp,  int channels[])
{
  int i;

  fprintf( fout, "Scan Start Time (s),");
  for( i = 0; i < dp->cjc.cjc_len; i++ ) {
    fprintf( fout, " CJC Value %d,", i + EX1048_FIRST_CJC_CHANNEL);
    if( dp->cjc_ticks.cjc_ticks_len )
      fprintf( fout, " CJC dt (100 ns) %d,", i + EX1048_FIRST_CJC_CHANNEL);
    if( dp->cjc_counts.cjc_counts_len )
      fprintf( fout, " CJC Raw (counts) %d,", i + EX1048_FIRST_CJC_CHANNEL);
  }

  for( i = 0; i < dp->x.x_len; i++ ) {
    fprintf( fout, " Value %d,", channels[i]);
    if( dp->x_ticks.x_ticks_len )
      fprintf( fout, " dt (100 ns) %d,", channels[i]);
    if( dp->x_counts.x_counts_len )
      fprintf( fout, " Raw (counts) %d,", channels[i]);
  }
  fprintf( fout, "\n");
  return 0;
}

/* dump scanlist's worth of data as a comma-separated-value record */
int fdump_data_page_as_csv( FILE *fout, EX1048_RPCSampleData *dp,  int channels[], EX1048_RPCEUConversion *conversions)
{
  int i;
  uint16_t *timestamps;

  fprintf( fout, "%8.8f,", 
	   (double) 
	   ( dp->ss_secs 
	     + ( (double )dp->ss_usecs / MICRO_SECS_PER_SEC ) ) );

  for( i = 0; i < dp->cjc.cjc_len; i++ ) {
    timestamps = ( uint16_t * )dp
      ->cjc_ticks.cjc_ticks_val;
    fprintf( fout, " " FP_VAL_STR ",", dp->cjc.cjc_val[i]);
    if( dp->cjc_ticks.cjc_ticks_len )
      fprintf( fout, " %10hu,", ntohs(timestamps[i]) );
    if( dp->cjc_counts.cjc_counts_len )
      fprintf( fout, "  %+8d,", dp->cjc_counts.cjc_counts_val[i]);
  }

  for( i = 0; i < dp->x.x_len; i++ ) {
    timestamps = ( uint16_t * )dp
      ->x_ticks.x_ticks_val;
    fprintf( fout, " ");
    fprintf( fout, build_float_format(conversions[channels[i]]), dp->x.x_val[i]);
    fprintf( fout, ",");
    if( dp->x_ticks.x_ticks_len )
      fprintf( fout, " %10hu,", ntohs(timestamps[i]) );
    if( dp->x_counts.x_counts_len )
      fprintf( fout, "  %+8d,", dp->x_counts.x_counts_val[i]);
  }
  fprintf( fout, "\n");
  return 0;
}


int readfifo_cmd( EX1048_CLIENT *cl, int num_pages, int *fifo_count, 
		  EX1048_RPCFIFOArray *rpc_result)
{
  EX1048_RPCFIFOPageCount rpc_page_count;
  enum clnt_stat result;
  
  memset(&rpc_page_count, 0, sizeof(rpc_page_count));
  
  rpc_page_count.lid = cl->lid;
  rpc_page_count.page_count = num_pages;
  
  result = read_fifo_0x00000101( rpc_page_count, rpc_result, cl->rpc_cl);
  if( result != RPC_SUCCESS ) {
    fprintf( stderr, "readfifo_cmd RPC Failed %d\n", result);
    return EX1048_ERROR_RPC_ERROR;
  } else if ( rpc_result->error_code != EX1048_SUCCESS ) {
    fprintf( stderr, "readfifo_cmd Failed %d\n", rpc_result->error_code );
    return rpc_result->error_code;
  } else {
    *fifo_count = rpc_result->fifo_count;
  }

  return rpc_result->error_code;
}


/*****************************************************************************
Revision History
 
$Log: readfifo.c,v $
Revision 1.6  2004/10/01 21:38:55  gbernhardt
fixed a bug that would cause readfifo to do very bad things in certain situations (bug #259)

Revision 1.5  2004/09/30 20:34:32  gbernhardt
minor fix to the previous bugfix

Revision 1.4  2004/09/30 19:16:25  gbernhardt
fixed formatting in readfifo client program to match that in the web interface (bug #283)

Revision 1.3  2004/09/25 19:42:49  gbernhardt
fixed error reporting (bug #259, although the bug itself isn't actually fixed yet)

Revision 1.2  2004/09/22 17:52:07  gbernhardt
merging alpha back onto the root (bug #79)

Revision 1.1.2.9  2004/09/21 14:34:26  gbernhardt
added link IDs (bug #227)

Revision 1.1.2.8  2004/08/30 19:32:48  gbernhardt
renamed 'msecs' variables on XDR structs etc. as 'ticks', and fixed some references to 'usec timestamps' to say '100ns timestamps' (bug #210)

Revision 1.1.2.7  2004/08/25 17:07:25  pmecklenburg
added support for limit values

Revision 1.1.2.6  2004/08/21 20:07:52  gbernhardt
overhauled RPC system to use new program number (bug #68)

Revision 1.1.2.5  2004/05/28 14:38:21  gbernhardt
output was missing a )

Revision 1.1.2.4  2004/05/25 20:12:40  gbernhardt
modified readfifo program and cgi to display nicely formatted times

Revision 1.1.2.3  2004/05/03 13:00:03  nbarendt
changed floating-point display value at Kevin Ballou's request - MattM

Revision 1.1.2.2  2004/04/20 22:07:38  nbarendt
moving files

Revision 1.1.2.1  2004/04/20 13:56:31  nbarendt
moved from lib dir

Revision 1.3.2.2  2004/04/19 14:41:23  nbarendt
updated output formatting

Revision 1.3.2.1  2004/04/13 20:04:03  nbarendt
added error return

Revision 1.3  2004/04/12 14:39:21  nbarendt
updated to support data output in CSV format
updated to support data output of raw counts in hexadecimal
fixed channel numbering in display

Revision 1.2  2004/03/12 16:46:48  nbarendt
modifed to reflect changes in client code, RPC server, and driver

Revision 1.1  2004/02/12 19:19:15  nbarendt
initial checkin


*****************************************************************************/
