/*****************************************************************************
Copyright VXI Technoloy, Inc., 2004
 
Author:     $Author: gbernhardt $    
Date:       $Date: 2004/09/22 17:52:07 $
Revision:   $Revision: 1.2 $
 
$Header: /home/shared/cvsroot/ex1048/uClinux-dist/user/ex1048/client/setconversions.c,v 1.2 2004/09/22 17:52:07 gbernhardt Exp $
 
*****************************************************************************/

/* command line program to set the current conversions */

#include<stdio.h>
#include<rpc/rpc.h>
#include<unistd.h>
#include<getopt.h>
#include<stdlib.h>
#include<ctype.h>
#include"ex1048.h"
#include"libex1048.h"

#include"ex1048_errors.h"

const char conv_names[16][CONV_NAME_LEN] = {
  "MV",
  "J",
  "K",
  "T",
  "E",
  "B",
  "S",
  "R",
  "N",
  "U0",
  "U1",
  "RESERVED1",
  "RESERVED2",
  "CJC",
  "CJC Data Returned",
  "MV_1X" };

int setconversions_main( int argc, char **argv )
{
  char *prgname;
  char optstring[]="tuh:v";
  char defaulthost[] = "localhost";
  char defaultnetid[] = "udp";
  EX1048_CLIENT *cl;
  int c;
  int i, j;
  EX1048_RPCresult result;
  int channels[EX1048_MAX_NUM_CHANNELS];
  int num_channels = 0;
  int num_channel_args;
  EX1048_RPCEUConversion conversions[EX1048_MAX_NUM_CHANNELS];
  int verbose = 0; /* default non-verbose */
  char *host;
  char *netid;
  char conv_str[CONV_NAME_LEN]; 
  int conv_found = 0;
  
  memset(&result, 0, sizeof(result));
  memset(conversions, 0, EX1048_MAX_NUM_CHANNELS * sizeof(EX1048_RPCEUConversion));
  
  /* default to localhost via UDP */
  prgname=argv[0];
  host = defaulthost;
  netid = defaultnetid;

  while ((c=getopt(argc,argv,optstring))!=-1) {
    switch(c) {
    case 't': netid = "tcp"; break;
    case 'u': netid = "udp"; break;
    case 'h': host = optarg; break;
    case 'v': verbose = 1; break;
    default: show_usage();
    }
  }

  /* make sure we have at least a channel and a conversion on the command line 
     remaining */
  if( ( argc - optind ) < 2 ) {
      show_usage();
  }

  num_channel_args = argc - optind - 1;

  for(i = 0; i < num_channel_args; i++ ) {
    char *end_ptr, *end_ptr2;
    int temp_ch;
    channels[num_channels] = strtol( argv[i+optind], &end_ptr, 0 );
    num_channels++;
    /* look for a range */
    if( *end_ptr == RANGE_DELIM ) {
      temp_ch = strtol( end_ptr+1, &end_ptr2, 0 );
      if( ( end_ptr != end_ptr2 ) 
	  && ( temp_ch > channels[num_channels - 1]) ) {
	/* the user has specified a range of channels */
	for( j = channels[num_channels - 1] + 1; j <=  temp_ch  ; j++ ) {
	  channels[num_channels] = j;
	  num_channels++;	  
	}
      } else {
	show_usage();
      }
    }
  }
  
  /* convert to all uppercase */
  strcpy( conv_str, argv[argc-1] );
  for( i = 0; i < strlen( conv_str ); i++)
    conv_str[i] = toupper( conv_str[i] );

  /* lookup conversion */
  for( i = 0; i <  sizeof( conv_names ) 
	 / ( sizeof(char) * CONV_NAME_LEN ) ; i++ ) {
    if( strcmp( conv_names[i], conv_str ) == 0 ) {
      conv_found = 1;
      conversions[0]= i;
      break;
    }
  }
  if( !conv_found ) {
    fprintf( stderr, "Unknown Conversion: %s\n", conv_str);
    fprintf( stderr, "Valid Conversions Sources: \n");
    for( i = 0; i < sizeof( conv_names ) / (sizeof(char) * CONV_NAME_LEN); i++ ) {
      fprintf(stderr, "%s\n", conv_names[i]);
    }
    show_usage();
  }
  
  
  if( strlen(argv[argc-1]) == 5 ) {
    if( strcmp( conv_str, "MV_1X" ) == 0 ) {
      conversions[0] = EX1048_RPC_EU_MV_1X;
    } else {
      show_usage();
    }
  } else if( strlen(argv[argc-1]) == 2  ) {
    if( strcmp( conv_str, "MV" ) == 0 )
      conversions[0] = EX1048_RPC_EU_MV;
    else if( strcmp( conv_str, "U0" ) == 0 )
      conversions[0] = EX1048_RPC_USER_DEFINED_0;
    else if( strcmp( conv_str, "U1" ) == 0 )
      conversions[0] = EX1048_RPC_USER_DEFINED_1;
    else
      show_usage();
  } else {
    switch( conv_str[0] ) {
    case 'J': conversions[0] = EX1048_RPC_EU_TC_J; break;
    case 'K': conversions[0] = EX1048_RPC_EU_TC_K; break;
    case 'T': conversions[0] = EX1048_RPC_EU_TC_T; break;
    case 'E': conversions[0] = EX1048_RPC_EU_TC_E; break;
    case 'B': conversions[0] = EX1048_RPC_EU_TC_B; break;
    case 'S': conversions[0] = EX1048_RPC_EU_TC_S; break;
    case 'R': conversions[0] = EX1048_RPC_EU_TC_R; break;
    case 'N': conversions[0] = EX1048_RPC_EU_TC_N; break;
    default:
      show_usage();
    }
  }
  /* replicate the zeroeth conversion */
  for( i = 1; i < num_channels; i++ ) {
    conversions[i] = conversions[0];
  }

  /* connect to the device */
  cl = ex1048_clnt_create( host, netid, EX1048PROGVERS, LID_NONE );

  /*  set the conversions  */
  if( setconversions_cmd( cl, num_channels, 
			  channels, conversions, &result ) == EX1048_SUCCESS )
    {
      if(verbose) printf(EX1048_CLIENT_SUCCESS_STR);
    }
  else
    {
      printf(EX1048_CLIENT_FAILURE_STR);
      fprintf( stderr,"Error Setting Conversions %d\n", result.error_code);
    }

  /* destroy the client handle */
  ex1048_clnt_destroy( cl );
  return 0;
}


int setconversions_cmd( EX1048_CLIENT *cl, int num_channels, int channels[], 
			EX1048_RPCEUConversion conversions[],
			EX1048_RPCresult  *rpc_result)
{
  EX1048_RPCConversionList *rpc_conversions;
  int i;

  /* setup the RPC structure */
  rpc_conversions = malloc( sizeof(EX1048_RPCConversionList) );
  if( rpc_conversions == NULL )
    {
      fprintf( stderr, "malloc error\n");
      exit(-1);
    }
  memset(rpc_conversions, 0, sizeof(EX1048_RPCConversionList));
  
  /* channels */
  rpc_conversions->channels.channels_len = num_channels;
  rpc_conversions->channels.channels_val = malloc( sizeof(int) 
						   * rpc_conversions->channels.channels_len);
  if( rpc_conversions->channels.channels_val == NULL )
    {
      fprintf( stderr,"malloc error\n");
      exit(-1);
    }

  for( i = 0; i < rpc_conversions->channels.channels_len; i++ )
    {
      rpc_conversions->channels.channels_val[i] = channels[i];
    }
  /* conversions */
  rpc_conversions->conversions.conversions_len = num_channels;
  rpc_conversions->conversions.conversions_val = malloc( sizeof(EX1048_RPCEUConversion) 
							 * rpc_conversions->conversions.conversions_len);
  if( rpc_conversions->conversions.conversions_val == NULL )
    {
      fprintf( stderr,"malloc error\n");
      exit(-1);
    }

  for( i = 0; i < rpc_conversions->conversions.conversions_len; i++ )
    {
      rpc_conversions->conversions.conversions_val[i] = conversions[i];
    }

  /* set the LID */
  rpc_conversions->lid = cl->lid;
  
  /* do the RPC */
  set_channel_conversions_0x00000101( *rpc_conversions, rpc_result, cl->rpc_cl);

  xdr_free( (xdrproc_t) xdr_EX1048_RPCConversionList, (char *) rpc_conversions );

  return rpc_result->error_code;
}

/*****************************************************************************
Revision History
 
$Log: setconversions.c,v $
Revision 1.2  2004/09/22 17:52:07  gbernhardt
merging alpha back onto the root (bug #79)

Revision 1.1.2.8  2004/09/21 14:34:26  gbernhardt
added link IDs (bug #227)

Revision 1.1.2.7  2004/08/21 20:07:52  gbernhardt
overhauled RPC system to use new program number (bug #68)

Revision 1.1.2.6  2004/07/08 20:21:18  gbernhardt
reordered the conversion types to correspond with changes in the FPGA

Revision 1.1.2.5  2004/07/07 16:22:19  gbernhardt
added user-defined conversions

Revision 1.1.2.4  2004/07/07 15:37:16  gbernhardt
replaced 100MV and 50MV conversions with MV_1X and MV

Revision 1.1.2.3  2004/04/21 14:25:24  nbarendt
added channel range support on command-line

Revision 1.1.2.2  2004/04/20 22:07:38  nbarendt
moving files

Revision 1.1.2.1  2004/04/20 13:56:31  nbarendt
moved from lib dir

Revision 1.1.2.1  2004/04/13 20:05:13  nbarendt
changed conversions supported; changed str compares

Revision 1.1  2004/03/12 16:47:40  nbarendt
initial checkin



*****************************************************************************/
