/*****************************************************************************
Copyright VXI Technoloy, Inc., 2004
 
Author:     $Author: gbernhardt $    
Date:       $Date: 2004/09/22 17:52:07 $
Revision:   $Revision: 1.2 $
 
$Header: /home/shared/cvsroot/ex1048/uClinux-dist/user/ex1048/client/setfreqs.c,v 1.2 2004/09/22 17:52:07 gbernhardt Exp $
 
*****************************************************************************/

/* command line program to set the current filter cut-off frequencies */

#include<stdio.h>
#include<rpc/rpc.h>
#include<unistd.h>
#include<getopt.h>
#include<stdlib.h>
#include<ctype.h>
#include"ex1048.h"
#include"libex1048.h"

#include"ex1048_errors.h"


int setfreqs_main( int argc, char **argv )
{
  char *prgname;
  char optstring[]="tuh:v";
  char defaulthost[] = "localhost";
  char defaultnetid[] = "udp";
  EX1048_CLIENT *cl;
  int c;
  int i, j;
  EX1048_RPCresult result;
  int channels[EX1048_MAX_NUM_CHANNELS];
  int num_channels = 0;
  int num_channel_args;
  double freqs[EX1048_MAX_NUM_CHANNELS];
  int verbose = 0; /* default non-verbose */
  char *host;
  char *netid;
  
  memset(&result, 0, sizeof(result));
  
  /* default to localhost via UDP */
  prgname=argv[0];
  host = defaulthost;
  netid = defaultnetid;

  while ((c=getopt(argc,argv,optstring))!=-1) {
    switch(c) {
    case 't': netid = "tcp"; break;
    case 'u': netid = "udp"; break;
    case 'h': host = optarg; break;
    case 'v': verbose = 1; break;
    default: show_usage();
    }
  }

  /* make sure we have at least a channel and a frequency on the command line 
     remaining */
  if( ( argc - optind ) < 2 ) {
      show_usage();
  }

  num_channel_args = argc - optind - 1;

  for(i = 0; i < num_channel_args; i++ ) {
    char *end_ptr, *end_ptr2;
    int temp_ch;
    channels[num_channels] = strtol( argv[i+optind], &end_ptr, 0 );
    num_channels++;
    /* look for a range */
    if( *end_ptr == RANGE_DELIM ) {
      temp_ch = strtol( end_ptr+1, &end_ptr2, 0 );
      if( ( end_ptr != end_ptr2 ) 
	  && ( temp_ch > channels[num_channels - 1]) ) {
	/* the user has specified a range of channels */
	for( j = channels[num_channels - 1] + 1; j <=  temp_ch  ; j++ ) {
	  channels[num_channels] = j;
	  num_channels++;	  
	}
      } else {
	show_usage();
      }
    }
  }

  freqs[0] = strtod( argv[ argc - 1], NULL );


  /* filter */
  for( i = 1; i < num_channels; i++ ) {
    freqs[i] = freqs[0];
  }

  /* connect to the device */
  cl = ex1048_clnt_create( host, netid, EX1048PROGVERS, LID_NONE );
  
  /*  set the conversions  */
  if( setfreqs_cmd( cl, num_channels, 
		    channels, freqs, &result ) == EX1048_SUCCESS ) {
    if(verbose) printf(EX1048_CLIENT_SUCCESS_STR);
  } else {
    printf(EX1048_CLIENT_FAILURE_STR);
    fprintf( stderr,"Error Setting Filter Frequencies %d\n", result.error_code);
  }

  /* destroy the client handle */
  ex1048_clnt_destroy( cl );
  return 0;
}


int setfreqs_cmd( EX1048_CLIENT *cl, int num_channels, int channels[], 
		  double freqs[], EX1048_RPCresult  *rpc_result)
{
  EX1048_RPCFilterList *rpc_freqs;
  int i;
  
  /* setup the RPC structure */
  rpc_freqs = malloc( sizeof(EX1048_RPCFilterList) );
  if( rpc_freqs == NULL )
    {
      fprintf( stderr, "malloc error\n");
      exit(-1);
    }
  memset(rpc_freqs, 0, sizeof(EX1048_RPCFilterList));
  
  /* channels */
  rpc_freqs->channels.channels_len = num_channels;
  rpc_freqs->channels.channels_val = malloc( sizeof(int) 
						   * rpc_freqs->channels.channels_len);
  if( rpc_freqs->channels.channels_val == NULL )
    {
      fprintf( stderr,"malloc error\n");
      exit(-1);
    }

  for( i = 0; i < rpc_freqs->channels.channels_len; i++ )
    {
      rpc_freqs->channels.channels_val[i] = channels[i];
    }
  /* frequencies */
  rpc_freqs->cutoff_freq.cutoff_freq_len = num_channels;
  rpc_freqs->cutoff_freq.cutoff_freq_val = malloc( sizeof(double) 
							 * rpc_freqs->cutoff_freq.cutoff_freq_len);
  if( rpc_freqs->cutoff_freq.cutoff_freq_val == NULL )
    {
      fprintf( stderr,"malloc error\n");
      exit(-1);
    }

  for( i = 0; i < rpc_freqs->cutoff_freq.cutoff_freq_len; i++ )
    {
      rpc_freqs->cutoff_freq.cutoff_freq_val[i] = freqs[i];
    }
  
  /* set the LID */
  rpc_freqs->lid = cl->lid;
  
  /* do the RPC */
  set_filt_freq_0x00000101( *rpc_freqs, rpc_result, cl->rpc_cl);

  xdr_free( (xdrproc_t) xdr_EX1048_RPCFilterList, (char *) rpc_freqs );

  return rpc_result->error_code;
}

/*****************************************************************************
Revision History
 
$Log: setfreqs.c,v $
Revision 1.2  2004/09/22 17:52:07  gbernhardt
merging alpha back onto the root (bug #79)

Revision 1.1.2.5  2004/09/21 14:34:27  gbernhardt
added link IDs (bug #227)

Revision 1.1.2.4  2004/08/21 20:07:52  gbernhardt
overhauled RPC system to use new program number (bug #68)

Revision 1.1.2.3  2004/04/21 14:25:24  nbarendt
added channel range support on command-line

Revision 1.1.2.2  2004/04/20 22:07:38  nbarendt
moving files

Revision 1.1.2.1  2004/04/20 13:56:31  nbarendt
moved from lib dir

Revision 1.1.2.1  2004/04/13 20:07:52  nbarendt
initial checkin




*****************************************************************************/
