/* command line program to set LED status */

#include<stdio.h>
#include<rpc/rpc.h>
#include<unistd.h>
#include<getopt.h>
#include<stdlib.h>
#include<ctype.h>
#include"ex1048.h"
#include"libex1048.h"

#define TEST_SCROLL  1
#define TEST_ALLON   2
#define TEST_ALLOFF  3
#define TEST_CHECKER 4
#define TEST_INVERT  5
#define TEST_ALL     6

void test_scroll (EX1048_CLIENT *cl, EX1048_RPCUserLEDStatus *status, EX1048_RPCresult *rpc_result) {
  int i, j;
  
  for (i=-5; i<=EX1048_MAX_NUM_CHANNELS; i++) {
    memset(status->leds, 0, EX1048_MAX_NUM_CHANNELS * sizeof(int));
    for (j=0; j<5; j++) {
      if (i + j >= 0 && i + j < EX1048_MAX_NUM_CHANNELS) {
	status->leds[i + j] = 1;
      }
    }
    setledstatus_cmd(cl, status, rpc_result);
  }
}

void test_allon (EX1048_CLIENT *cl, EX1048_RPCUserLEDStatus *status, EX1048_RPCresult *rpc_result) {
  int i;
  
  status->error_code = 0;
  for (i=0; i<EX1048_MAX_NUM_CHANNELS; i++) {
    status->leds[i] = 1;
  }
  setledstatus_cmd(cl, status, rpc_result);
}

void test_alloff (EX1048_CLIENT *cl, EX1048_RPCUserLEDStatus *status, EX1048_RPCresult *rpc_result) {
  int i;
  
  status->error_code = 0;
  for (i=0; i<EX1048_MAX_NUM_CHANNELS; i++) {
    status->leds[i] = 0;
  }
  setledstatus_cmd(cl, status, rpc_result);
}

void test_checker (EX1048_CLIENT *cl, EX1048_RPCUserLEDStatus *status, EX1048_RPCresult *rpc_result) {
  int i;
  
  status->error_code = 0;
  for (i=0; i<EX1048_MAX_NUM_CHANNELS; i++) {
    status->leds[i] = ((i + i / 16) % 2 == 0 ? 1 : 0);
  }
  setledstatus_cmd(cl, status, rpc_result);
}

void test_invert (EX1048_CLIENT *cl, EX1048_RPCUserLEDStatus *status, EX1048_RPCresult *rpc_result) {
  int i;
  
  getledstatus_cmd(cl, status);
  if (status->error_code) {
    return;
  }
  status->error_code = 0;
  for (i=0; i<EX1048_MAX_NUM_CHANNELS; i++) {
    /* invert each led */
    status->leds[i] = !status->leds[i];
  }
  setledstatus_cmd(cl, status, rpc_result);
}

/*
  testfn
*/
static int testfn (EX1048_CLIENT *cl, int test) {  
  struct EX1048_RPCUserLEDStatus status;
  struct EX1048_RPCresult rpc_result;
  
  memset(&status, 0, sizeof(status));
  memset(&rpc_result, 0, sizeof(rpc_result));
  
  switch (test) {
  case TEST_SCROLL:
    test_alloff(cl, &status, &rpc_result);
    test_scroll(cl, &status, &rpc_result);
    break;
  case TEST_ALLON:
    test_allon(cl, &status, &rpc_result);
    break;
  case TEST_ALLOFF:
    test_alloff(cl, &status, &rpc_result);
    break;
  case TEST_CHECKER:
    test_checker(cl, &status, &rpc_result);
    break;
  case TEST_INVERT:
    test_invert(cl, &status, &rpc_result);
    break;
  case TEST_ALL:
  default:
    while (1) {
      test_alloff(cl, &status, &rpc_result);
      test_scroll(cl, &status, &rpc_result);
      test_scroll(cl, &status, &rpc_result);
      sleep(1);
      test_allon(cl, &status, &rpc_result);
      sleep(1);
      test_alloff(cl, &status, &rpc_result);
      sleep(1);
      test_checker(cl, &status, &rpc_result);
      sleep(1);
      test_invert(cl, &status, &rpc_result);
      sleep(1);
      test_invert(cl, &status, &rpc_result);
      sleep(1);
      test_invert(cl, &status, &rpc_result);
      sleep(1);
    }
    break;
  }
  
  return 0;
}

/*
  parse_arguments
*/
static int parse_arguments (int arg_count, char **args, EX1048_CLIENT *cl) {
  int result;
  struct EX1048_RPCUserLEDStatus *status;
  struct EX1048_RPCresult rpc_result;
  int i;
  
  memset(&rpc_result, 0, sizeof(rpc_result));
  
  /* allocate the LED status RPC structure */
  status = (struct EX1048_RPCUserLEDStatus *)malloc(sizeof(struct EX1048_RPCUserLEDStatus));
  if (!status) {
    fprintf(stderr, "Error: malloc failed while setting LED status\n");
    return -1;
  }
  memset(status, 0, sizeof(struct EX1048_RPCUserLEDStatus));
  
  /* populate the RPC structure. */
  for (i=0; i<arg_count; i++) {
    status->leds[i] = atoi(args[i]);
  }
  
  /* do the RPC */
  result = setledstatus_cmd(cl, status, &rpc_result);
  
  free(status);
  return result;
}

int setledstatus_main (int argc, char **argv) {
  const char optstring[] = "T:tuh:v";
  const char defaulthost[] = "localhost";
  const char defaultnetid[] = "udp";
  char *host = (char *)defaulthost;
  char *netid = (char *)defaultnetid;
  int verbose = 0;
  int test = 0;
  int c;
  EX1048_CLIENT *cl;
  EX1048_RPCresult rpc_result;
  
  memset(&rpc_result, 0, sizeof(rpc_result));
  
  /* defaults to localhost via UDP */
  while ((c=getopt(argc, argv, optstring)) != -1) {
    switch(c) {
    case 't':
      netid = "tcp";
      break;
    case 'u':
      netid = "udp";
      break;
    case 'h':
      host = optarg;
      break;
    case 'v':
      verbose = 1;
      break;
    case 'T':
      if (optarg) {
	test = atoi(optarg);
      } else {
	test = TEST_ALL;
      }
      break;
    default:
      show_usage();
      break;
    }
  }
  
  /* make sure the user set at least 1 LED status */
  if (!test && ((argc - optind) < 1)) {
    show_usage();
  }
  
  /* create the client handle */
  cl = ex1048_clnt_create(host, netid, EX1048PROGVERS, LID_NONE);
  
  if (test) {
    testfn(cl, test);
  } else {
    /* get the arguments */
    if (parse_arguments(argc - optind, argv + optind, cl) == EX1048_SUCCESS) {
      if (verbose) {
	printf(EX1048_CLIENT_SUCCESS_STR);
      }
    } else {
      printf(EX1048_CLIENT_FAILURE_STR);
      fprintf(stderr, "Error setting LED status (error = %d)\n", rpc_result.error_code);
    }
  }

  /* destroy the client handle */
  ex1048_clnt_destroy(cl);
  
  return 0;
}

int setledstatus_cmd (EX1048_CLIENT *cl, EX1048_RPCUserLEDStatus *status, EX1048_RPCresult *rpc_result) {
  /* set the LID */
  status->lid = cl->lid;
  
  /* do the RPC */
  set_led_status_0x00000101(*status, rpc_result, cl->rpc_cl);
  
  return rpc_result->error_code;
}
