/*******************************************************************************
 *
 * Module Name: dmmdevices.c
 *
 * Copyright 2006 VXI Technology Inc. as an unpublished work.
 * All rights reserved.
 *
 * The information contained herein is confidential property of VXI Technology,
 * Inc.  The use, copying, transfer, or disclosure of such information is
 * prohibited except by express written agreement with VXI Technology, Inc.
 *
 *
 * Module Description:
 *   This is the EX1629 DMM devices library.
 *
 ******************************************************************************/

#include "dmmlib.h"
#include "dmmdevices.h"

/**********************************************************/
/* Keithley 2000 */
/**********************************************************/

cmd_list_t keithley_2000_volt_init = {
  "*RST\n",
  ":INIT:CONT 0\n", /* continuous mode must be off to use the READ? command */
  ":SENS:FUNC 'VOLT:DC'\n",
  ":SENS:VOLT:DC:NPLC 10\n",
  NULL
};

cmd_list_t keithley_2000_res_init = {
  "*RST\n",
  ":INIT:CONT 0\n", /* continuous mode must be off to use the READ? command */
  ":SENS:FUNC 'FRES'\n",
  ":SENS:FRES:AVER 1\n",
  ":SENS:FRES:AVER:TCON  REP\n",
  ":SENS:FRES:AVER:COUN 10\n",
  NULL  
};

cmd_list_t keithley_2000_read = {
  "READ?\n",
  NULL
};

struct dmm_cmds keithley_2000_cmds = {
  keithley_2000_volt_init,
  keithley_2000_res_init,
  ":SENS:VOLT:DC:RANG",
  ":SENS:FRESISTANCE:RANGE",
  keithley_2000_read,
  "*IDN?\n",
  "%255[^,],%255[^,],%255[^,],%255[^,\n]\n",
  "cal:prot:date?;ndue?\n",
  5000
};


/**********************************************************/
/* Keithley 2001/2002 */
/**********************************************************/

cmd_list_t keithley_2001_volt_init = {
  "*RST\n",
  ":INIT:CONT 0\n", /* continuous mode must be off to use the READ? command */
  ":SENS:FUNC 'VOLT:DC'\n",
  ":SENS:VOLT:DC:NPLC 10\n",
  NULL
};

cmd_list_t keithley_2001_res_init = {
  "*RST\n",
  ":INIT:CONT 0\n", /* continuous mode must be off to use the READ? command */
  ":SENS:FUNC 'FRES'\n",
  ":SENS:FRES:AVER 1\n",
  ":SENS:FRES:AVER:TCON  REP\n",
  ":SENS:FRES:AVER:COUN 10\n",
  ":SENS:FRES:OCOM 1\n",
  NULL
};

cmd_list_t keithley_2001_read = {
  "READ?\n",
  NULL
};

struct dmm_cmds keithley_2001_cmds = {
  keithley_2000_volt_init,
  keithley_2000_res_init,
  ":SENS:VOLT:DC:RANG",
  ":SENS:FRESISTANCE:RANGE",
  keithley_2000_read,
  "*IDN?\n",
  "%255[^,],%255[^,],%255[^,],%255[^,\n]\n",
  "cal:prot:date?;ndue?\n",
  5000
};


/**********************************************************/
/* Agilent (HP) 3458A */
/**********************************************************/
cmd_list_t agilent_3458A_volt_init = {
  "CLEAR;",
  "RESET;",
  "INBUF ON;",
/*   ":INIT:CONT 0\n", /\* continuous mode must be off to use the READ? command *\/ */
/*   ":SENS:FUNC 'VOLT:DC'\n", */
/*   ":SENS:VOLT:DC:NPLC 10\n", */
  "NPLC 10;",
  "NRDGS 1;",
  NULL
};

cmd_list_t agilent_3458A_res_init = {
  "RESET;",
  "INBUF ON;",
  "OHMF;",  //set to OHMS
  "NPLC 100;", //Number of Power-Line cycles: 10
  "OCOMP ON;", //Offset compensation on
  "NRDGS 1;", //Number of readings: 10
  NULL
};


cmd_list_t agilent_3458A_read = {
  "TRIG;\n",
  //"RMATH MEAN;\n", //gets the mean, theoretically.
  NULL
};

struct dmm_cmds agilent_3458A_cmds = {
  agilent_3458A_volt_init,
  agilent_3458A_res_init,
  "RANGE ",
  "RANGE ",
  agilent_3458A_read,
  "ID?;\n",
  "%2[^,0123456789\r\n]%255[^,\r\n]\r\n",
  "CALSTR?;",
  5000 
};

/**********************************************************/
/* Agilent 34401A */
/**********************************************************/

cmd_list_t agilent_34401A_volt_init = {
  "*RST\n",
  ":SENS:FUNC 'VOLT:DC'\n",
  ":SENS:VOLT:DC:NPLC 10\n",
  "SAMP:COUN 1\n",
  "TRIG:SOUR IMM\n",
  NULL
};

cmd_list_t agilent_34401A_res_init = {
 "*RST\n",
 "SENS:FUNC 'FRES'\n",
 "SENS:FRES:NPLC 100\n",
 "SAMP:COUN 1\n",
 "TRIG:SOUR IMM\n",
 NULL
};

cmd_list_t agilent_34401A_read = {
  "INIT\n",
  "READ?\n",
  NULL
};

struct dmm_cmds agilent_34401A_cmds = {
  agilent_34401A_volt_init,
  agilent_34401A_res_init,
  ":SENS:VOLT:DC:RANG",
  ":SENS:FRES:RANG",
  agilent_34401A_read,
  "*IDN?\n",
  "%255[^,],%255[^,],%255[^,],%255[^,\n]\n",
  "cal:str?\n",
  5000
};

/**********************************************************/
/* Agilent 34410A/34411A */
/**********************************************************/

cmd_list_t agilent_34411A_volt_init = {
  "*RST\n",
  ":SYST:LANG '34401A'\n",
  ":SENS:FUNC 'VOLT:DC'\n",
  ":SENS:VOLT:DC:NPLC 10\n",
  ":SAMP:COUN 1\n",
  ":TRIG:SOUR IMM\n",
  NULL
};

cmd_list_t agilent_34411A_res_init = {
 "*RST\n",
 ":SYST:LANG '34401A'\n",
 ":SENS:FUNC 'FRES'\n",
 ":SENS:FRES:NPLC 100\n",
 ":SAMP:COUN 1\n",
 ":TRIG:SOUR IMM\n",
 NULL
};

cmd_list_t agilent_34411A_read = {
  "INIT\n",
  "READ?\n",
  NULL
};

struct dmm_cmds agilent_34411A_cmds = {
  agilent_34411A_volt_init,
  agilent_34411A_res_init,
  ":SENS:VOLT:DC:RANG",
  ":SENS:FRES:RANG",
  agilent_34411A_read,
  "*IDN?\n",
  "%255[^,],%255[^,],%255[^,],%255[^,\n]\n",
  "cal:str?\n",
  5000
};
/**********************************************************/
/* Device Commands */
/**********************************************************/

struct device g_devices[] = {
  {0, "Keithley 2000", &keithley_2000_cmds},
  {0, "Keithley 2001/2002", &keithley_2001_cmds},
  {0, "Agilent(HP) 3458A", &agilent_3458A_cmds},
  {0, "Agilent(HP) 34401A", &agilent_34401A_cmds},
  {0, "Agilent(HP) 34410/34411A",&agilent_34411A_cmds},
  {0, NULL, NULL}
};
