/*
  dmmlib.h
*/

#ifndef __DMMLIB_H
#define __DMMLIB_H

#include "scpilib.h"
#include "dmmdevices.h"

//enum here needs to line up with g_devices in dmmdevices.c
enum {
  DMM_DEVICE_MIN = 0,
  DMM_DEVICE_KEITHLEY_2000 = DMM_DEVICE_MIN,
  DMM_DEVICE_KEITHLEY_2001,
  DMM_DEVICE_AGILENT_3458A,
  DMM_DEVICE_AGILENT_34401A,
  DMM_DEVICE_AGILENT_34411A,
  DMM_DEVICE_MAX = DMM_DEVICE_AGILENT_34411A
};

/* this size is also hard-coded in the dmm_get_device_info function's sscanf call */
#define DMM_MAX_SCPI_STRING_LENGTH 256

struct dmm_handle {
  struct scpi_handle *scpi;
  int device;
};

struct dmm_params {
  int device;
  char *host;
  char *gpib_addr;
};

struct dmm_device_info {
  char manufacturer[DMM_MAX_SCPI_STRING_LENGTH + 1];
  char model[DMM_MAX_SCPI_STRING_LENGTH + 1];
  char serial[DMM_MAX_SCPI_STRING_LENGTH + 1];
  char version[DMM_MAX_SCPI_STRING_LENGTH + 1];
};

struct dmm_date {
  int year;
  int month;
  int day;
};

struct dmm_cal_info {
  struct dmm_date date;
  struct dmm_date due;
  char calstr[DMM_MAX_SCPI_STRING_LENGTH + 1];
};



int dmm_get_device_count (void);
const char * dmm_get_device_name (int index);

int dmm_init_start(const struct dmm_params **params,struct dmm_handle **dmm,struct device **device);
struct dmm_handle * dmm_volt_init (const struct dmm_params *data);
struct dmm_handle * dmm_res_init (const struct dmm_params *data);
int dmm_set_range_init(struct dmm_handle *handle, struct device **device);
int dmm_set_range_finish(struct dmm_handle *handle,char** buf);
int dmm_set_volt_range (struct dmm_handle *handle, double value);
int dmm_set_res_range (struct dmm_handle *handle, double value);
int dmm_read (struct dmm_handle *handle, double *value);
int dmm_get_device_info (struct dmm_handle *handle, struct dmm_device_info *devinfo);
int dmm_get_cal_info (struct dmm_handle *handle, struct dmm_cal_info *cal_info);
int dmm_destroy (struct dmm_handle *handle);


//old files
/*int dmm_get_device_count (void);
const char * dmm_get_device_name (int index);

struct dmm_handle * dmm_init (const struct dmm_params *data);
int dmm_set_range (struct dmm_handle *handle, double value);
int dmm_read (struct dmm_handle *handle, double *value);
int dmm_get_device_info (struct dmm_handle *handle, struct dmm_device_info *devinfo);
int dmm_get_cal_info (struct dmm_handle *handle, struct dmm_cal_info *cal_info);
int dmm_destroy (struct dmm_handle *handle);
*/
#endif
