#include "dmmlib.h"
#include "unistd.h"

void show_usage(int argc, char **argv)
{
  int i;

  printf("Usage:  %s: <device> <ip> <gpib> <range> \n", argv[0]);
  printf("\t<device> is a numeral one of:\n");
  for (i=0; i<dmm_get_device_count(); i++) {
    printf("\t\t%d: %s\n", i, dmm_get_device_name(i));
  }
  printf("\t<ip> is the ip address or hostname of the GPIB/LAN gateway\n");
  printf("\t<gpib> is the gpib address DMM\n");
  printf("\t<range> is the range to use for the voltage reading\n");
}

int main (int argc, char **argv) {
  struct dmm_handle *dmm;
  struct dmm_params params;
  struct dmm_device_info info;
  struct dmm_cal_info cal_info;
  double value;
  int device_number;
  double volt_range;
  
  /* check command-line arguments */
  if( argc != 5 ) {
    show_usage( argc, argv);
    return -1;
  }
  
  printf("Got to beginning\n");
  
  device_number = atoi(argv[1]);

  if( ( device_number < 0 ) || device_number > (dmm_get_device_count() - 1) ) {
    show_usage( argc, argv);
    return -2;
  }
  
  printf("Device # ok\n");
  
  params.device = device_number;
  params.host = argv[2];
  params.gpib_addr = argv[3];

  volt_range = atof( argv[4]);

/*   params.device = DMM_DEVICE_AGILENT_3458A;  */
/*   params.host = "10.1.2.160"; */
/*   params.gpib_addr = "gpib0,22"; */
  
  
  dmm = dmm_volt_init(&params);
  if (!dmm) {
    printf("failed to init\n");
    return 1;
  }
  
  printf("Finished init\n");

  if (dmm_set_volt_range(dmm, volt_range) != 0) {
    printf("failed to set range\n");
    return 1;
  }

  printf("Set range ok\n");
  
  if (dmm_get_device_info(dmm, &info) != 0) {
    printf("failed to get device info\n");
    return 1;
  }

  printf("Device info ok\n");
  
  printf("Device info: %s : %s : %s : %s\n", info.manufacturer, info.model, info.serial, info.version);

  if (dmm_read(dmm, &value) != 0) {
    printf("failed to read value\n");
    return 1;
  }

  printf("value: %f\n", value);
  
  
  if (dmm_get_cal_info(dmm, &cal_info) != 0) {
    printf("failed to get cal info\n");
    return 1;
  }
  
  printf("Cal info: %s (cal %i,%i,%i) (due %i,%i,%i)\n",
	 cal_info.calstr, 
	 cal_info.date.year, cal_info.date.month, cal_info.date.day,
	 cal_info.due.year, cal_info.due.month, cal_info.due.day);
  
  dmm_destroy(dmm);
  
  return 0;
}
