/*****************************************************************************
Copyright VXI Technoloy, Inc., 2004
 
Author:     $Author: gbernhardt $    
Date:       $Date: 2004/10/28 20:16:26 $
Revision:   $Revision: 1.7 $
 
$Header: /home/shared/cvsroot/ex1048/uClinux-dist/user/ex1048/shared/ex1048.x,v 1.7 2004/10/28 20:16:26 gbernhardt Exp $
 
*****************************************************************************/


%/* instrument specific constants */
const EX1048_MAX_SCANLIST_LENGTH 	=       48;
const EX1048_MIN_SCANLIST_LENGTH 	=        1;
const EX1048_MAX_NUM_CHANNELS           =       48;
const EX1048_FIRST_CJC_CHANNEL          =       48;
/* for testing - allow all 63 channels */
/*const EX1048_MAX_CJC_CHANNELS	        =       12;*/
const EX1048_MAX_CJC_CHANNELS	        =       16;
/*const EX1048_MAX_DATA                   =       60;*/
const EX1048_MAX_DATA                   =       63;
const EX1048_MIN_CHANNEL                =        0;
const EX1048_MAX_CHANNEL                =       47;
const EX1048_MAX_TS_BYTES               =       96; /* 2 bytes/ts * 48 channels */
/*const EX1048_MAX_CJC_TS_BYTES           =       24; *//* 2 bytes/ts * 12 CJC channels */
const EX1048_MAX_CJC_TS_BYTES           =       32; /* 2 bytes/ts * 12 CJC channels */
const EX1048_MAX_LIMIT_INTS		=	8;  /* 64 bits * 2 (upper/lower) * 2 (sets) */
%#define EX1048_MIN_SAMPLING_PERIOD_SECS     0.001 /* minimum period of 1000us (1KHz) */
%#define EX1048_MAX_SAMPLING_PERIOD_SECS    4294.0 /* NAB??? what is the max period */
%#define EX1048_SAMPLING_PERIOD_RES_SECS     0.0005 /* NAB??? what is the resolution */

const EX1048_MAX_BULK_TEST_DATA_LEN	=	16384;
const EX1048_MAX_FIFO_RESPONSE          =          1000;/* NAB??? */

%/* macros for extracting interface number and "real" version number from
%   RPC version numbers */
%#define EXTRACT_RPC_INTERFACE(v) ((v >> 8) & 0xFFFF)
%#define EXTRACT_RPC_VERSION(v) (v & 0xFF)

%/* All EX1048 RPC functions will return an error code as the first
%   element in their result, or as just a result structure.
%
%*/
const EX1048_SUCCESS			=	0;

struct EX1048_RPCresult
{
	int error_code;
};

%/* the scanlist id is not currently used, but for support of extended features
% in the future, we will build in support now.
% To query the current scanlist, use the following value */
const EX1048_CURRENT_SCANLIST = -1;

struct EX1048_RPCscanlist 
{
	int error_code;
	int lid;
	int id;
	int channels<EX1048_MAX_SCANLIST_LENGTH>; 
};

%#define LIMITS_0_MIN_OFFSET 0
%#define LIMITS_0_MAX_OFFSET 2
%#define LIMITS_1_MIN_OFFSET 4
%#define LIMITS_1_MAX_OFFSET 6

struct EX1048_RPCSampleData
{
	/* timestamp - scan start time in seconds and microseconds */
	unsigned int ss_secs;
	unsigned int ss_usecs;
	float x<EX1048_MAX_NUM_CHANNELS>;   /* 32-bit floating point measurement values */
	int x_counts<EX1048_MAX_NUM_CHANNELS>; /* 20-bit ADC Readings */
	opaque x_ticks<EX1048_MAX_TS_BYTES>; /* 16-bit measurement timestamps - 100ns offsets 
						from scan start time */
	float cjc<EX1048_MAX_CJC_CHANNELS>; /* 32-bit floating point CJC values */
	int cjc_counts<EX1048_MAX_CJC_CHANNELS>; /* 20-bit ADC Readings */
	opaque cjc_ticks<EX1048_MAX_CJC_TS_BYTES>; /* 16-bit CJC timestamps - 100ns offsets 
						from scan start time */

	/* limit data 4 bits for each channel */
	unsigned int limits<EX1048_MAX_LIMIT_INTS>;
};

typedef EX1048_RPCSampleData* EX1048_RPCSampleData_p;

%#define LID_NONE (-1)

typedef int EX1048_RPCLinkID;

struct EX1048_RPCFIFOArray
{
	int error_code;
	/* number of fifo entries remaining after this response */
	unsigned int fifo_count; 
	EX1048_RPCSampleData_p fifo<EX1048_MAX_FIFO_RESPONSE>;
};

enum EX1048_RPCEUConversion
{
	EX1048_RPC_EU_MIN            = 0,
	EX1048_RPC_EU_MV             = EX1048_RPC_EU_MIN,
	EX1048_RPC_EU_TC_J           = 1,
	EX1048_RPC_EU_TC_K           = 2, 
	EX1048_RPC_EU_TC_T           = 3,
	EX1048_RPC_EU_TC_E           = 4,
	EX1048_RPC_EU_TC_B           = 5,
	EX1048_RPC_EU_TC_S           = 6,
	EX1048_RPC_EU_TC_R           = 7, 
	EX1048_RPC_EU_TC_N           = 8,
	EX1048_RPC_USER_DEFINED_0    = 9,
	EX1048_RPC_USER_DEFINED_1    = 10,
	EX1048_RPC_RESERVED1         = 11,
	EX1048_RPC_RESERVED2         = 12,
	EX1048_RPC_CJC               = 13,
	EX1048_RPC_CJC_DATA_RETURNED = 14,
	EX1048_RPC_EU_MV_1X          = 15,
	EX1048_RPC_EU_MAX            = EX1048_RPC_USER_DEFINED_1
/*	EX1048_RPC_EU_MAX            = EX1048_RPC_EU_50MV*/
};

struct EX1048_RPCConversionList
{
	int error_code;
	int lid;
	int channels<EX1048_MAX_NUM_CHANNELS>;
	EX1048_RPCEUConversion conversions<EX1048_MAX_NUM_CHANNELS>;
};

struct EX1048_RPCFilterList
{
	int error_code;
	int lid;
	int channels<EX1048_MAX_NUM_CHANNELS>;
	double cutoff_freq<EX1048_MAX_NUM_CHANNELS>;
};

/* this should match the temperatures defined in uClinux-dist/user/ex1048/driver/acqmain.h */
enum EX1048_RPCTemperature
{
	EX1048_TEMP_UNITS_MIN = 0,
	EX1048_TEMP_UNITS_C   = EX1048_TEMP_UNITS_MIN,
	EX1048_TEMP_UNITS_F   = 1,
	EX1048_TEMP_UNITS_MAX = EX1048_TEMP_UNITS_F
};

struct EX1048_RPCFIFOFormat
{
	int error_code;
	int lid;
	/* 0 = no cjc reporting, 1 = cjc reporting, -1 = no change */
	int report_cjc;
	/* 0 = no timestamp reporting, 1 = timestamp  reporting, -1 = no change */
	int report_timestamp;
	/* 0 = no count reporting, 1 =  count reporting, -1 = no change */
	int report_count;
	/* between TEMP_UNITS_MIN and TEMP_UNITS_MAX, inclusive */
	int temp_units;
	/* 0 = not blocking, 1 = blocking */
	int blocking_mode;
};

struct EX1048_RPCTrigCount
{
	int error_code;
	int lid;
	unsigned int count;
};

struct EX1048_RPCTrigDelay
{
	int error_code;
	int lid;
	double delay_sec;
	double resolution;
};

struct EX1048_RPCTrigInf
{
	int error_code;
	int lid;
	int inf;
};

struct EX1048_RPCTrigTimer
{
	int error_code;
	int lid;
	double timer_period;
	double timer_resolution;
};

const EX1048_NUM_VTB_MASKS = 4;
const EX1048_NUM_DIO_MASKS = 4;
struct EX1048_RPCTrigSrc
{
	int error_code;
	int lid;
	unsigned char vtb_masks[EX1048_NUM_VTB_MASKS];
	unsigned char dio_masks[EX1048_NUM_DIO_MASKS];
	int timer_enable;
	int software;
	int imm;
};

struct EX1048_RPCByte
{
	int error_code;
	int lid;
	unsigned char value;
};

const MAX_LIMIT_SETS = 2;
struct EX1048_RPCLimits
{
	int error_code;
	int lid;
	int set;
	int channels<EX1048_MAX_NUM_CHANNELS>;
	float lower<EX1048_MAX_NUM_CHANNELS>;
	float upper<EX1048_MAX_NUM_CHANNELS>;
};

struct EX1048_RPCLimitEnables
{
	int error_code;
	int lid;
	int enables[EX1048_MAX_NUM_CHANNELS];
};

struct EX1048_RPCcontinuous
{
	int error_code;
	int lid;
	int cont;
};

const MAX_CONVERSION_SETS = 2;
const MAX_CONVERSION_COEFFICIENTS = 13;
struct EX1048_RPCUserConversion
{
	int error_code;
	int lid;
	int set;
	float forward_coeff[MAX_CONVERSION_COEFFICIENTS];
	float inverse_coeff[MAX_CONVERSION_COEFFICIENTS];
};

const MAX_CAL_HOST_LENGTH = 16; /* 255.255.255.255 */
const MAX_CAL_GPIB_ADDR_LENGTH = 32;
%#define FACTORY_CAL_KEY 0x31415926
struct EX1048_RPCFactoryCalibrationData
{
	int error_code;
	int lid;
	int device;
	int key;
	string host<>;
	string gpib_addr<>;
/*	float cjc_offset[12]; */
	int nfs_enable;
	string nfs_hostname<>;
	string nfs_share<>;
	string nfs_options<>;
};

struct EX1048_RPCCalibrationStatus
{
	int error_code;
	int lid;
	int percent_completed;
};

struct EX1048_RPCInterface
{
	int iface;
	int versions<>;
};

struct EX1048_RPCVersions
{
	int error_code;
	int lid;
	EX1048_RPCInterface interfaces<>;
};

const MAX_FIRMWARE_STRING_LENGTH = 64;
struct EX1048_RPCFirmwareVersion
{
	int error_code;
	int lid;
	string prefix<>;
	int major;
	int minor;
	int build;
};

struct EX1048_RPCCJCTemperatures {
	int error_code;
	int lid;
	int channels<EX1048_MAX_NUM_CHANNELS>;
	float temperatures<EX1048_MAX_NUM_CHANNELS>;
};

struct EX1048_RPCCJCEnables {
	int error_code;
	int lid;
	int channels<EX1048_MAX_NUM_CHANNELS>;
	int enables<EX1048_MAX_NUM_CHANNELS>;
};

const MAX_SYSTEM_ERROR_LENGTH = 2048;
struct EX1048_RPCErrorStatus {
	int error_code;
	int lid;
	string error_string<MAX_SYSTEM_ERROR_LENGTH>;
};

struct EX1048_RPCCalibrationRunning {
	int error_code;
	int lid;
	int cal_running;
};

struct EX1048_RPCSelfTestResult {
	int error_code;
	int lid;
	int result_code;
	string result_string<>;
};

struct EX1048_RPCSystemStatus {
	int error_code;
	int lid;
	
	int trig_idle;
	int trig_initialized;
	int trig_arm_active;
	int trig_trig_active;
	int trig_device_layer_active;
	int trig_waiting_for_arm;
	int trig_waiting_for_trigger;
	int trig_acquiring;

	int fifo_pages;
	int fifo_count;	
	int fifo_blocking_mode_enabled;
	int fifo_block_status;
	
	int cal_running;
	
	string error_status<MAX_SYSTEM_ERROR_LENGTH>;
};

const MAX_LIMIT_SETS = 2;
const NUM_LIMIT_WORDS = 2;
struct EX1048_RPCLimitBitmask {
	unsigned int bitwords[NUM_LIMIT_WORDS];
};
struct EX1048_RPCLimitSet {
	struct EX1048_RPCLimitBitmask masks[2]; /* 'min' and 'max' */
};
struct EX1048_RPCLimitStatus {
	int error_code;
	int lid;
	struct EX1048_RPCLimitSet limits[MAX_LIMIT_SETS];
};

const MAX_DIO_BITS = 8;
const LIMIT_EVENT_SET_0_MIN = 1;
const LIMIT_EVENT_SET_0_MAX = 2;
const LIMIT_EVENT_SET_1_MIN = 4;
const LIMIT_EVENT_SET_1_MAX = 8;
struct EX1048_RPCDIOLimitChannelEvents {
	int dio_bits[MAX_DIO_BITS];
};
struct EX1048_RPCDIOLimitEvents {
	int error_code;
	int lid;
	EX1048_RPCDIOLimitChannelEvents channel_events[EX1048_MAX_NUM_CHANNELS];
};

struct EX1048_RPCDIOLimitFlags {
	int error_code;
	int lid;
	unsigned int pull_low[MAX_DIO_BITS];
	unsigned int latch[MAX_DIO_BITS];
};

struct EX1048_RPCDIOLimitEnables {
	int error_code;
	int lid;
	unsigned int bit_enables[MAX_DIO_BITS];
};

struct EX1048_RPCUserLEDStatus {
	int error_code;
	int lid;
	int leds[EX1048_MAX_NUM_CHANNELS];
};

struct EX1048_RPCUserAuxLEDStatus {
	int error_code;
	int lid;
	int aux;
};

struct EX1048_RPCStoredSelfCal {
	int error_code;
	int lid;
	int has_stored_cal;
};

enum EX1048_RPCSingleChannelUnit
{
	EX1048_CHANNEL_UNITS_MIN     = 0,
	EX1048_CHANNEL_UNITS_DEGC    = EX1048_CHANNEL_UNITS_MIN,
	EX1048_CHANNEL_UNITS_DEGF    = 1,
	EX1048_CHANNEL_UNITS_VOLTS   = 2,
	EX1048_CHANNEL_UNITS_UNKNOWN = 3,
	EX1048_CHANNEL_UNITS_MAX     = EX1048_CHANNEL_UNITS_UNKNOWN
};

struct EX1048_RPCChannelUnits {
	int error_code;
	int lid;
	EX1048_RPCSingleChannelUnit units[EX1048_MAX_NUM_CHANNELS];
};

struct EX1048_RPCCalibrationDate {
	int error_code;
	int lid;
	int stamp; /* unix timestamp */
	string date_string<>; /* date as a string */
	int sec; /* 0-60 */
	int min; /* 0-60 */
	int hour; /* 0-23 */
	int day; /* 1-31 */
	int month; /* 0-11 */
	int year; /* eg 2004 */
};

struct EX1048_RPCSerialNumber {
	int error_code;
	int lid;
	string serial<>;
};

struct EX1048_RPCLockStatus {
	int error_code;
	int lid;
	int locked;
};

struct EX1048_RPCSelfCalResult {
	int error_code;
	int lid;
	int magic;    /* if uptime was too low, pass this in to override */
	int recommended_uptime;
	int current_uptime;
};

struct EX1048_RPCLink {
	int error_code;
	int lid;
};

struct EX1048_RPCScanlistID {
	int error_code;
	int lid;
	int id;
};

struct EX1048_RPCFIFOPageCount {
	int error_code;
	int lid;
	int page_count;
};

struct EX1048_RPCLimitSetNumber {
	int error_code;
	int lid;
	int set;
};

struct EX1048_RPCConversionSetNumber {
	int error_code;
	int lid;
	int set;
};

struct EX1048_RPCSelfCalMagic {
	int error_code;
	int lid;
	int magic;
};

struct EX1048_RPCPort {
	int error_code;
	int lid;
	unsigned short port;
};

program VXITECH_PROG
{
	version COREVERS
	{
		/* VERSION REQUEST FUNCTIONS */
		EX1048_RPCVersions GET_RPC_VERSION( void ) = 0x200;
		EX1048_RPCFirmwareVersion GET_FIRMWARE_VERSION( void ) = 0x201;

		/* ERROR STATUS FUNCTIONS */
		EX1048_RPCErrorStatus GET_ERROR_STATUS( void ) = 0x300;
	} = 0x00000001;
} = 100416;

program VXITECH_PROG
{
	version EX1048PROGVERS 
	{
		/* RESET FUNCTION */
		EX1048_RPCresult RESET( EX1048_RPCLinkID ) = 0x100;

		/* SCANLIST FUNCTIONS */
		EX1048_RPCresult SET_SCANLIST( EX1048_RPCscanlist )  = 0x200;
		EX1048_RPCscanlist GET_SCANLIST( EX1048_RPCScanlistID ) = 0x201;
	
		/* EU CONVERSION FUNCTIONS */
		EX1048_RPCresult SET_CHANNEL_CONVERSIONS( EX1048_RPCConversionList )  = 0x300;
		EX1048_RPCConversionList GET_CHANNEL_CONVERSIONS( EX1048_RPCLinkID )  = 0x301;

		/* CHANNEL UNIT FUNCTIONS */
		EX1048_RPCChannelUnits GET_CHANNEL_UNITS( EX1048_RPCLinkID ) = 0x400;

		/* FILTER FUNCTIONS */
		EX1048_RPCresult SET_FILT_FREQ( EX1048_RPCFilterList  )  = 0x500;
		EX1048_RPCFilterList GET_FILT_FREQ( EX1048_RPCLinkID )  = 0x501;

		/* FIFO FUNCTIONS */
		EX1048_RPCresult RESET_FIFO( EX1048_RPCLinkID ) = 0x600;
		EX1048_RPCFIFOArray READ_FIFO( EX1048_RPCFIFOPageCount ) = 0x601;
		EX1048_RPCFIFOFormat SET_FIFO_CONFIG(  EX1048_RPCFIFOFormat  ) = 0x602;

	/* TRIGGER BLOCK */

		/* INIT FUNCTIONS */
		EX1048_RPCresult TRIG_ABORT( EX1048_RPCLinkID ) = 0x700;
		EX1048_RPCresult TRIG_INIT_CONT_SET( EX1048_RPCcontinuous ) = 0x701;
		EX1048_RPCcontinuous TRIG_INIT_CONT_GET( EX1048_RPCLinkID ) = 0x702;
		EX1048_RPCresult TRIG_INIT( EX1048_RPCLinkID ) = 0x703;
		EX1048_RPCresult TRIG_TIMER_PERIOD_SET( EX1048_RPCTrigTimer ) = 0x704;
		EX1048_RPCTrigTimer TRIG_TIMER_PERIOD_GET( EX1048_RPCLinkID ) = 0x705;

		/* TRIGGER FUNCTIONS */
		EX1048_RPCresult TRIG_TRIG_COUNT_SET( EX1048_RPCTrigCount ) = 0x710;
		EX1048_RPCTrigCount TRIG_TRIG_COUNT_GET( EX1048_RPCLinkID ) = 0x711;
		EX1048_RPCresult TRIG_TRIG_SOURCE_SET( EX1048_RPCTrigSrc ) = 0x712;
		EX1048_RPCTrigSrc TRIG_TRIG_SOURCE_GET( EX1048_RPCLinkID ) = 0x713;
		EX1048_RPCresult TRIG_TRIG_DELAY_SET( EX1048_RPCTrigDelay ) = 0x714;
		EX1048_RPCTrigDelay TRIG_TRIG_DELAY_GET( EX1048_RPCLinkID ) = 0x715;
		EX1048_RPCresult TRIG_TRIG_INF_SET( EX1048_RPCTrigInf ) = 0x716;
		EX1048_RPCTrigInf TRIG_TRIG_INF_GET( EX1048_RPCLinkID ) = 0x717;
		
		/* ARM FUNCTIONS */
		EX1048_RPCresult TRIG_ARM_COUNT_SET( EX1048_RPCTrigCount ) = 0x720;
		EX1048_RPCTrigCount TRIG_ARM_COUNT_GET( EX1048_RPCLinkID ) = 0x721;
		EX1048_RPCresult TRIG_ARM_SOURCE_SET( EX1048_RPCTrigSrc ) = 0x722;
		EX1048_RPCTrigSrc TRIG_ARM_SOURCE_GET( EX1048_RPCLinkID ) = 0x723;
		EX1048_RPCresult TRIG_ARM_DELAY_SET( EX1048_RPCTrigDelay ) = 0x724;
		EX1048_RPCTrigDelay TRIG_ARM_DELAY_GET( EX1048_RPCLinkID ) = 0x725;
		EX1048_RPCresult TRIG_ARM_INF_SET( EX1048_RPCTrigInf ) = 0x726;
		EX1048_RPCTrigInf TRIG_ARM_INF_GET( EX1048_RPCLinkID ) = 0x727;

		/* SOFTWARE ARM AND TRIGGER FUNCTIONS */
		EX1048_RPCresult TRIG_SOFTWARE_ARM( EX1048_RPCLinkID ) = 0x730;
		EX1048_RPCresult TRIG_SOFTWARE_TRIGGER( EX1048_RPCLinkID ) = 0x731;

		/* TRIGGER RESET */
		EX1048_RPCresult TRIG_RESET( EX1048_RPCLinkID ) = 0x0740;

	/* END TRIGGER BLOCK */

		/* DIGITAL IO FUNCTIONS */
		EX1048_RPCByte GET_DIO_INPUT( EX1048_RPCLinkID ) = 0x800;
		EX1048_RPCByte GET_DIO_OUTPUT( EX1048_RPCLinkID ) = 0x801;
		EX1048_RPCresult SET_DIO_OUTPUT( EX1048_RPCByte ) = 0x802;
		EX1048_RPCByte GET_DIO_ENABLE( EX1048_RPCLinkID ) = 0x803;
		EX1048_RPCresult SET_DIO_ENABLE( EX1048_RPCByte ) = 0x804;
		EX1048_RPCresult SEND_DIO_PULSE( EX1048_RPCByte ) = 0x805;

		/* TRIGGER BUS FUNCTIONS */
		EX1048_RPCByte GET_VTB_INPUT( EX1048_RPCLinkID ) = 0x900;
		EX1048_RPCByte GET_VTB_OUTPUT( EX1048_RPCLinkID ) = 0x901;
		EX1048_RPCresult SET_VTB_OUTPUT( EX1048_RPCByte ) = 0x902;
		EX1048_RPCByte GET_VTB_ENABLE( EX1048_RPCLinkID ) = 0x903;
		EX1048_RPCresult SET_VTB_ENABLE( EX1048_RPCByte ) = 0x904;
		EX1048_RPCresult SEND_VTB_PULSE( EX1048_RPCByte ) = 0x905;

		/* STREAMING FUNCTIONS */
		EX1048_RPCresult STREAMING_CONNECT( EX1048_RPCPort ) = 0xA00;
		EX1048_RPCresult STREAMING_DISCONNECT( EX1048_RPCLinkID ) = 0xA01;

		/* LIMIT FUNCTIONS */
		EX1048_RPCresult SET_LIMITS( EX1048_RPCLimits ) = 0xB00;
		EX1048_RPCLimits GET_LIMITS( EX1048_RPCLimitSetNumber ) = 0xB01;
		EX1048_RPCLimitStatus GET_LIMIT_STATUS( EX1048_RPCLinkID ) = 0xB02;
		EX1048_RPCresult SET_LIMIT_ENABLES( EX1048_RPCLimitEnables ) = 0xB03;
		EX1048_RPCLimitEnables GET_LIMIT_ENABLES( EX1048_RPCLinkID ) = 0xB04;
		EX1048_RPCresult SET_DIO_LIMIT_EVENTS( EX1048_RPCDIOLimitEvents ) = 0xB05;
		EX1048_RPCDIOLimitEvents GET_DIO_LIMIT_EVENTS( EX1048_RPCLinkID ) = 0xB06;
		EX1048_RPCresult SET_DIO_LIMIT_FLAGS( EX1048_RPCDIOLimitFlags ) = 0xB07;
		EX1048_RPCDIOLimitFlags GET_DIO_LIMIT_FLAGS( EX1048_RPCLinkID ) = 0xB08;
		EX1048_RPCDIOLimitEnables GET_DIO_LIMIT_ENABLES( EX1048_RPCLinkID ) = 0xB09;

		/* USER-DEFINED CONVERSION FUNCTIONS */
		EX1048_RPCresult SET_USER_CONV( EX1048_RPCUserConversion ) = 0xC00;
		EX1048_RPCUserConversion GET_USER_CONV( EX1048_RPCConversionSetNumber ) = 0xC01;

		/* USER-DEFINED CJC FUNCTIONS */
		EX1048_RPCresult SET_USER_CJC_TEMPERATURES( EX1048_RPCCJCTemperatures ) = 0xD00;
		EX1048_RPCCJCTemperatures GET_USER_CJC_TEMPERATURES( EX1048_RPCLinkID ) = 0xD01;
		EX1048_RPCresult SET_USER_CJC_ENABLES( EX1048_RPCCJCEnables ) = 0xD02;
		EX1048_RPCCJCEnables GET_USER_CJC_ENABLES( EX1048_RPCLinkID ) = 0xD03;

		/* LED STATUS FUNCTIONS */
		EX1048_RPCUserLEDStatus GET_LED_STATUS( EX1048_RPCLinkID ) = 0xE00;
		EX1048_RPCresult SET_LED_STATUS( EX1048_RPCUserLEDStatus ) = 0xE01;
		EX1048_RPCUserAuxLEDStatus GET_AUX_LED_STATUS( EX1048_RPCLinkID ) = 0xE02;
		EX1048_RPCresult SET_AUX_LED_STATUS( EX1048_RPCUserAuxLEDStatus ) = 0xE03;

		/* FACTORY CALIBRATION FUNCTIONS */
		EX1048_RPCresult FACTORY_CAL_INIT( EX1048_RPCFactoryCalibrationData ) = 0xF00;
		EX1048_RPCCalibrationDate GET_CALIBRATION_DATE( EX1048_RPCLinkID ) = 0xF01;

		/* SELF CALIBRATION FUNCTIONS */
		EX1048_RPCSelfCalResult SELF_CALIBRATION_INIT( EX1048_RPCSelfCalMagic ) = 0x1000;
		EX1048_RPCresult SELF_CALIBRATION_CLEAR( EX1048_RPCLinkID ) = 0x1001;
		EX1048_RPCresult SELF_CALIBRATION_STORE( EX1048_RPCLinkID ) = 0x1002;
		EX1048_RPCresult SELF_CALIBRATION_LOAD( EX1048_RPCLinkID ) = 0x1003;
		EX1048_RPCStoredSelfCal SELF_CALIBRATION_IS_STORED( EX1048_RPCLinkID ) = 0x1004;
		EX1048_RPCresult SELF_CALIBRATION_CLEAR_STORED( EX1048_RPCLinkID ) = 0x1005;

		/* CALIBRATION STATUS FUNCTIONS */
		EX1048_RPCCalibrationStatus GET_CALIBRATION_STATUS( EX1048_RPCLinkID ) = 0x1100;
		EX1048_RPCCalibrationRunning GET_CALIBRATION_RUNNING( EX1048_RPCLinkID ) = 0x1101;

		/* LOCKING FUNCTIONS */
		EX1048_RPCresult LOCK( EX1048_RPCLinkID ) = 0x1200;
		EX1048_RPCresult UNLOCK( EX1048_RPCLinkID ) = 0x1201;
		EX1048_RPCLockStatus CHECK_LOCK( EX1048_RPCLinkID ) = 0x1202;
		EX1048_RPCresult BREAK_LOCK( EX1048_RPCLinkID ) = 0x1203;

		/* SELF TEST FUNCTIONS */
		EX1048_RPCSelfTestResult SELF_TEST( EX1048_RPCLinkID ) = 0x1300;

		/* SYSTEM STATUS FUNCTIONS */
		EX1048_RPCSystemStatus GET_SYSTEM_STATUS( EX1048_RPCLinkID ) = 0x1400;
		EX1048_RPCSerialNumber GET_SYSTEM_SERIAL_NUMBER( EX1048_RPCLinkID ) = 0x1401;

		/* LINK ID FUNCTIONS */
		EX1048_RPCLink CREATE_LINK( void ) = 0x1500;
		EX1048_RPCresult DESTROY_LINK( EX1048_RPCLink ) = 0x1501;
	} = 0x00000101;
} = 100416;

/*****************************************************************************
Revision History
 
$Log: ex1048.x,v $
Revision 1.7  2004/10/28 20:16:26  gbernhardt
increased maximum timer interval to 4294 seconds (bug #355)

Revision 1.6  2004/10/26 20:37:10  gbernhardt
changed the maximum timer rate from 2kHz to 1kHz (bug #351)

Revision 1.5  2004/10/17 20:38:17  pmecklenburg
added self cal clear stored rpc

Revision 1.4  2004/10/15 20:21:33  gbernhardt
fixed DIO limit event semantics (bug #314)

Revision 1.3  2004/09/30 19:34:42  pmecklenburg
added factory cal key support

Revision 1.2  2004/09/22 17:52:14  gbernhardt
merging alpha back onto the root (bug #79)

Revision 1.1.2.53  2004/09/21 14:34:29  gbernhardt
added link IDs (bug #227)

Revision 1.1.2.52  2004/09/08 21:45:42  gbernhardt
added support for limit events to the driver, RPCs, and CGI (bug #168)

Revision 1.1.2.51  2004/09/07 21:59:19  pmecklenburg
added magic (uptime) to self cal init

Revision 1.1.2.50  2004/09/07 19:51:30  pmecklenburg
removed cjc_offset from factory cal

Revision 1.1.2.49  2004/08/31 20:27:25  gbernhardt
renamed a field on the EX1048_RPCInterface RPC structure to remove a conflict in the Windows driver

Revision 1.1.2.48  2004/08/30 19:32:49  gbernhardt
renamed 'msecs' variables on XDR structs etc. as 'ticks', and fixed some references to 'usec timestamps' to say '100ns timestamps' (bug #210)

Revision 1.1.2.47  2004/08/30 15:18:05  gbernhardt
added enable flags to channel 0 of the limits (they don't do anything yet) (bug #147)

Revision 1.1.2.46  2004/08/27 20:43:53  gbernhardt
modified trigger configuration to not occasionally break on valid configurations (bug #187)

Revision 1.1.2.45  2004/08/27 19:56:07  gbernhardt
overhauled the VXI11 server to support the spec better (bug #166)

Revision 1.1.2.44  2004/08/25 17:08:32  pmecklenburg
added support for perscan limit values

Revision 1.1.2.43  2004/08/23 21:39:58  pmecklenburg
added get_system_serial_number and get_calibration_date

Revision 1.1.2.42  2004/08/23 17:05:15  gbernhardt
removed old unused constants for RPC ports

Revision 1.1.2.41  2004/08/22 17:03:42  gbernhardt
renumbered RPC functions to make more sense with the new 2-program RPC interface (bug #184)

Revision 1.1.2.40  2004/08/21 20:07:54  gbernhardt
overhauled RPC system to use new program number (bug #68)

Revision 1.1.2.39  2004/08/18 20:28:01  pmecklenburg
added nfs log support to factory cal

Revision 1.1.2.38  2004/08/18 14:37:29  gbernhardt
added an RPC and userland app to get the channel units (bug #145)

Revision 1.1.2.37  2004/08/15 20:16:40  nbarendt
modified limit bit mask to use RPC data types

Revision 1.1.2.36  2004/08/13 19:42:09  pmecklenburg
added self cal is stored rpc

Revision 1.1.2.35  2004/08/13 18:00:18  gbernhardt
added a build number to the firmware version (bug #125)

Revision 1.1.2.34  2004/08/12 18:07:44  nbarendt
per bug 112, removed function parameter names

Revision 1.1.2.33  2004/08/10 19:55:51  gbernhardt
added support for aux LED (bug #86)

Revision 1.1.2.32  2004/08/10 18:32:23  gbernhardt
added driver interface for controlling LEDs (bug #86)

Revision 1.1.2.31  2004/08/09 17:17:07  pmecklenburg
added USER_OFFSET/cjc values

Revision 1.1.2.30  2004/08/09 15:26:16  gbernhardt
added 'system status' and 'limit status' RPCs (bugs #20 and #62)

Revision 1.1.2.29  2004/08/06 14:46:35  pmecklenburg
changed systemstatus to errorstatus

Revision 1.1.2.28  2004/08/04 19:07:56  gbernhardt
reworked the 'firmware version' RPC call (bug #64)

Revision 1.1.2.27  2004/08/03 21:39:07  gbernhardt
added software arm and trigger

Revision 1.1.2.26  2004/08/03 20:17:10  pmecklenburg
fixed locks, added calibrationrunning

Revision 1.1.2.25  2004/08/01 17:14:28  gbernhardt
added initial (incomplete) self-test

Revision 1.1.2.24  2004/07/30 17:47:01  gbernhardt
added preliminary VTB support (bug #56)

Revision 1.1.2.23  2004/07/29 21:47:51  pmecklenburg
added get_system_status rpc

Revision 1.1.2.22  2004/07/29 18:03:36  gbernhardt
added blocking mode control to config_fifo (bug #50)

Revision 1.1.2.21  2004/07/29 16:57:13  gbernhardt
added temperature units setting (bug #18)

Revision 1.1.2.20  2004/07/28 22:01:47  gbernhardt
added user-defined CJC temperature RPC (bug #37)

Revision 1.1.2.19  2004/07/26 18:22:09  gbernhardt
added firmware version and RPC version RPC calls (bug #46)

Revision 1.1.2.18  2004/07/26 14:48:41  gbernhardt
added support for multiple limit sets

Revision 1.1.2.17  2004/07/19 21:07:03  gbernhardt
added calibration status RPC

Revision 1.1.2.16  2004/07/19 20:31:47  gbernhardt
initial implementation of locking (doesn't do anything yet - just stubs)

Revision 1.1.2.15  2004/07/16 20:35:29  gbernhardt
modified setuserconv RPC and userland to only take one argument

Revision 1.1.2.14  2004/07/16 20:12:16  nbarendt
Per Bug 35
replaced magic number on VTB and DIO mask array lengths with constant

Revision 1.1.2.13  2004/07/15 19:56:40  gbernhardt
added factory cal

Revision 1.1.2.12  2004/07/14 20:51:46  nbarendt
Per Bug 30:  removed parameter name on SET_USER_CONV

Revision 1.1.2.11  2004/07/14 19:53:08  pmecklenburg
fixed TRIG_TRIG_INF_GET procedure number

Revision 1.1.2.10  2004/07/09 19:45:55  nbarendt
added server port numbers and added min scanlist lenght

Revision 1.1.2.9  2004/07/08 20:23:03  gbernhardt
reordered the conversion types to correspond with changes in the FPGA

Revision 1.1.2.8  2004/07/07 15:34:15  gbernhardt
added user-defined conversions, and replaced 100MV and 50MV conversions with MV_1X and MV

Revision 1.1.2.7  2004/06/28 18:52:59  gbernhardt
merged alpha alpha-trigger

Revision 1.1.2.6.2.2  2004/06/24 14:47:29  pmecklenburg
changed delay from int (clock ticks) to double (sec)

Revision 1.1.2.6.2.1  2004/06/22 18:59:23  pmecklenburg
added new RPC calls for updated trigger model

Revision 1.1.2.6  2004/06/17 14:11:31  gbernhardt
added set_limits and get_limits RPCs

Revision 1.1.2.5  2004/06/11 19:52:45  nbarendt
parameter names from function signatures so that the
windows rpcgen program doesn't barf on them

Revision 1.1.2.4  2004/06/11 17:11:24  pmecklenburg
added userland streaming server

Revision 1.1.2.3  2004/06/03 20:07:08  gbernhardt
added digital IO RPC and userland tools

Revision 1.1.2.2  2004/05/03 13:41:20  nbarendt
changed max EU setting

Revision 1.1.2.1  2004/04/20 13:24:12  nbarendt
moved from parent

Revision 1.4.2.2  2004/04/20 12:40:10  nbarendt
added support for raw ADC counts

Revision 1.4.2.1  2004/04/13 19:55:35  nbarendt
added several new interfaces

Revision 1.4  2004/04/12 13:55:23  nbarendt

added EX1048_FIRST_CJC_CHANNEL const
added raw counts to fifo data structure
added server reset function

Revision 1.3  2004/03/12 16:39:58  nbarendt
modified RPC structures; removed special functions/structures for returning data with timestamps (one general purpose sstructure is used now); added several RPC functions

Revision 1.2  2004/02/12 19:08:14  nbarendt
some improvements


*****************************************************************************/
