/*****************************************************************************
Copyright VXI Technoloy, Inc., 2003

Author: 	  $Author: gbernhardt $	
Date:		    $Date: 2004/12/10 19:23:38 $
Revision:   $Revision: 1.6 $

$Header: /home/shared/cvsroot/ex1048/uClinux-dist/user/ex1048/shared/ex1048_errors.c,v 1.6 2004/12/10 19:23:38 gbernhardt Exp $

*****************************************************************************/
#include<stdarg.h>
#include"ex1048_errors.h"

static EX1048_stringpair EX1048_errors[] =
  {
    {EX1048_ERROR_VXI11_NO_ERROR, "No Error"},
    {EX1048_ERROR_VXI11_SYNTAX_ERROR, "Syntax Error"},
    {EX1048_ERROR_VXI11_DEVICE_NOT_ACCESSIBLE, "Device Not Accessible"},
    {EX1048_ERROR_VXI11_INVALID_LINK_IDENTIFIER, "Invalid Link Identifier"},
    {EX1048_ERROR_VXI11_PARAMETER_ERROR, "Parameter Error"},
    {EX1048_ERROR_VXI11_CHANNEL_NOT_ESTABLISHED, "Channel Not Established"},
    {EX1048_ERROR_VXI11_OPERATION_NOT_SUPPORTED, "Operation Not Supported"},
    {EX1048_ERROR_VXI11_OUT_OF_RESOURCES, "Out of Resources"},
    {EX1048_ERROR_VXI11_DEVICE_LOCKED_BY_ANOTHER_LINK, "Device Locked By Another Link"},
    {EX1048_ERROR_VXI11_NO_LOCK_HELD_BY_THIS_LINK, "No Lock Held By This Link"},
    {EX1048_ERROR_VXI11_IO_TIMEOUT, "IO Timeout"},
    {EX1048_ERROR_VXI11_IO_ERROR, "IO Error"},
    {EX1048_ERROR_VXI11_INVALID_ADDRESS, "Invalid Address"},
    {EX1048_ERROR_VXI11_ABORT, "Abort"},
    {EX1048_ERROR_VXI11_CHANNEL_ALREADY_ESTABLISHED, "Channel Already Established"},
    
    {EX1048_SUCCESS,                       "Success"},
    {EX1048_ERROR_INVALID_SCANLIST_LENGTH, "Invalid Scanlist Length"},
    {EX1048_ERROR_INVALID_CHANNEL,         "Invalid Channel"},
    {EX1048_ERROR_INVALID_SAMPLING_RATE,   "Invalid Sampling Rate"},
    {EX1048_ERROR_INVALID_NULL_POINTER,    "NULL Pointer"},
    {EX1048_ERROR_INVALID_MALLOC,          "Dynamic Memory Allocation Failed"},
    {EX1048_ERROR_INVALID_SCANLIST_ID,     "Invalid Scanlist ID"},
    {EX1048_ERROR_INVALID_CONV_LIST_LENGTH,"Invalid Conversion List Length"},
    {EX1048_ERROR_INVALID_EU_CONVERSION,   "Invalid EU Conversion"},
    {EX1048_ERROR_REQUIRED_MODULE_MISSING, "Required Kernel Module Missing"},
    {EX1048_ERROR_OPENING_DEVICE_FILE,     "Error opening device file"},
    {EX1048_ERROR_DEVICE_ERROR,            "Device Error"},
    {EX1048_ERROR_RPC_ERROR,               "RPC Error"},

    {EX1048_ERROR_INVALID_TRIG_COUNT, "Invalid Trigger Count"},
    {EX1048_ERROR_INVALID_TRIG_TIMER_PERIOD, "Invalid Trigger Timer Period"},
    {EX1048_ERROR_INVALID_FILT_LIST_LENGTH, "Invalid Filter List Length"},
    {EX1048_ERROR_INVALID_FILT_FREQ, "Invalid Filter Frequency"},
    {EX1048_ERROR_ALREADY_STREAMING, "Already Streaming"},
    {EX1048_ERROR_SOCKET, "Socket Error"},
    {EX1048_ERROR_CONNECT, "Connection Error"},
    {EX1048_ERROR_NOT_CONNECTED, "Not Connected"},
    {EX1048_ERROR_INVALID_NUM_ENTRIES, "Invalid Number of Entries"},
    {EX1048_ERROR_NEGATIVE_TIME, "Negative Time"},
    {EX1048_ERROR_DEVICE_LOCKED, "Device Locked"},
    {EX1048_ERROR_CAL_IN_PROGRESS, "Calibration in Progress"},
    {EX1048_ERROR_SYSTEM_FAILURE, "System Failure"},
    {EX1048_ERROR_DEVICE_NOT_LOCKED, "Device Not Locked"},
    {EX1048_ERROR_INVALID_TRIGGER_CONFIGURATION, "Invalid Trigger Configuration"},
    {EX1048_ERROR_CALIBRATION_MISSING, "Calibration Missing"},
    {EX1048_ERROR_INIT_LAYER_NOT_ACTIVE, "Init Layer Not Active"},
    {EX1048_ERROR_ARM_LAYER_NOT_ACTIVE, "Arm Layer Not Active"},
    {EX1048_ERROR_TRIG_LAYER_NOT_ACTIVE, "Trigger Layer Not Active"},
    {EX1048_ERROR_INSUFFICIENT_UPTIME, "Insufficient Uptime"},
    {EX1048_ERROR_OUT_OF_LINK_IDS, "Out of Link IDs"},
    {EX1048_ERROR_INVALID_LID, "Invalid Link ID"},
    {EX1048_ERROR_DUPLICATE_SCANLIST_CHANNEL, "Duplicate Scanlist Channel"},
    {EX1048_ERROR_INVALID_FACTORY_KEY, "Invalid Factory Key"},
    {EX1048_ERROR_NO_STATUS, "Error no Status"},
    {EX1048_ERROR_NONVOL_SELF_CAL_MISSING, "Nonvolatile Self Cal Data Not Present"},
    {EX1048_ERROR_SELF_CAL_MISSING, "Current Self Cal Data Not Present"},
   { 0, (char*) NULL } /* this is the list terminator - only string portion is important */
  };

static char EX1048_unknown_err[] = "Unknown Error";

/*****************************************************************************
  Function: ex1048_set_error

  Returns   void
 
  Input 
         EX1048_result *result     - variable to record error in 
         EX1048_result error_code  - error code 
         char *msg                 - optional additional error message

  Description: 
    This function sets the error code for the variable pointed to be result.
    It is meant to be a centralized error reporting function.  Error logging
    can be performed in here.
*****************************************************************************/
void ex1048_set_error( int *result, int error_code, ... )
{
  va_list args;
  char *format;

  va_start(args, error_code);

  if( result == NULL )
    return;
  
  *result = error_code;

  format = va_arg( args, char *);

  /* NAB ??? add error logging here */
  /* send the error code and optional msg to stderr */
  fprintf( stderr, "EX1048SERVER: %s ", ex1048_get_error_msg(error_code) );
  vfprintf( stderr, format, args );
  fprintf( stderr, "\n");

  va_end(args);
}




/*****************************************************************************
  Function: ex1048_get_error_msg

  Returns   const pointer to the error string or NULL if error not found
 
  Input 
        EX1048_result result - error code to lookup

  Description: 
   This function looks up the error_code provided and returns the associated
   error message, or a default message if the error code is not found.
*****************************************************************************/
const char *ex1048_get_error_msg( int error_code )
{
  char *result = EX1048_unknown_err;
  int32 i;

  /* search through the error list for the provided error code, 
     stopping when a match is found, or the end of the list is located */
  for( i = 0; EX1048_errors[i].msg; i++ )
    {
      if( EX1048_errors[i].error == error_code )
	{
	  result = EX1048_errors[i].msg;
	  break;
	}
    }

  return result;
}

/*****************************************************************************
Revision History

$Log: ex1048_errors.c,v $
Revision 1.6  2004/12/10 19:23:38  gbernhardt
removed redundant EX1048_ERROR_NOTE_LOCKED error code (bug #366)

Revision 1.5  2004/10/17 20:38:47  pmecklenburg
added nonvol self cal missing and self cal missing errors

Revision 1.4  2004/10/13 20:20:09  pmecklenburg
added EX1048_ERROR_NOT_LOCKED

Revision 1.3  2004/09/30 19:36:02  pmecklenburg
removed EX1048_ERROR_OPENING_STATUS_FILE and added CAL_IN_PROGRESS, INVALID_FACTORY_KEY and NO_STATUS

Revision 1.2  2004/09/22 17:52:14  gbernhardt
merging alpha back onto the root (bug #79)

Revision 1.1.2.3  2004/09/09 17:47:00  gbernhardt
unified VXI11 and EX1048 errors (bug #234)

Revision 1.1.2.2  2004/04/21 14:23:43  nbarendt
changed get_error_msg to return an default
error message if the error code can't be found

Revision 1.1.2.1  2004/04/20 13:38:27  nbarendt
moved from parent dir

Revision 1.2  2004/03/12 16:40:52  nbarendt
added new error codes/messages

Revision 1.1  2004/02/12 19:18:40  nbarendt
initial checkin



*****************************************************************************/
